@@ -18,7 +18,7 @@ we would be doing something wrong.
 *********************************************************************/
 
 #ifdef _WIN32
-#include <windows.h>		// don't know why this is needed
+#include <WinSock2.h> // also includes <windows.h> // needed for sql.h whihc uses say HWND, DWORD, etc...
 #endif
 
 #include <sql.h>		/* ODBC header files */
@@ -43,7 +43,7 @@ static int current_driver;
  * provide no information until you actually run the query.
  * Prepare is not enough.
  * The openfirst variable tells us whether we are running in that mode. */
-static eb_bool openfirst = eb_false;
+static bool openfirst = false;
 
 #define SQL_MONEY 100
 
@@ -71,7 +71,7 @@ static const char *stmt_text = 0;	/* text of the SQL statement */
 static SQLRETURN rc;
 static const short *exclist;	/* list of error codes trapped by the application */
 static short translevel;
-static eb_bool badtrans;
+static bool badtrans;
 
 /* Through globals, make error info available to the application. */
 int rv_lastStatus, rv_stmtOffset;
@@ -194,11 +194,11 @@ static int errTranslate(const char *code)
 
 static char errorText[200];
 
-static eb_bool errorTrap(const char *cxerr)
+static bool errorTrap(const char *cxerr)
 {
 	short i, waste;
 	char errcodes[6];
-	eb_bool firstError, errorFound;
+	bool firstError, errorFound;
 
 	/* innocent until proven guilty */
 	rv_lastStatus = 0;
@@ -206,7 +206,7 @@ static eb_bool errorTrap(const char *cxerr)
 	rv_stmtOffset = 0;
 	rv_badToken = 0;
 	if (!rc)
-		return eb_false;	/* no problem */
+		return false;	/* no problem */
 
 	/* log the SQL statement that elicitted the error */
 	showStatement();
@@ -216,10 +216,10 @@ static eb_bool errorTrap(const char *cxerr)
 		    ("@ODBC fails to recognize one of the handles (env, connect, stmt)");
 
 	/* get error info from ODBC */
-	firstError = eb_true;
-	errorFound = eb_false;
+	firstError = true;
+	errorFound = false;
 
-	while (eb_true) {
+	while (true) {
 		rc = SQLError(henv, hdbc, hstmt,
 			      errcodes, &rv_vendorStatus, errorText,
 			      sizeof(errorText), &waste);
@@ -227,7 +227,7 @@ static eb_bool errorTrap(const char *cxerr)
 			if (firstError) {
 				printf
 				    ("ODBC command failed, but SQLError() provided no additional information\n");
-				return eb_true;
+				return true;
 			}
 			return errorFound;
 		}
@@ -236,13 +236,13 @@ static eb_bool errorTrap(const char *cxerr)
 		if (stringEqual(errcodes, "01S01"))
 			continue;
 
-		firstError = eb_false;
+		firstError = false;
 		if (cxerr && stringEqual(cxerr, errcodes))
 			continue;
 
 		if (errorFound)
 			continue;
-		errorFound = eb_true;
+		errorFound = true;
 		rv_lastStatus = errTranslate(errcodes);
 
 		/* Don't know how to get statement ofset or invalid token from ODBC.
@@ -346,23 +346,23 @@ Connect and disconect to SQL databases.
 
 /* disconnect from the database.  Return true if
  * an error occurs that is trapped by the application. */
-static eb_bool disconnect(void)
+static bool disconnect(void)
 {
 	stmt_text = 0;
 	hstmt = SQL_NULL_HSTMT;
 
 	if (!sql_database)
-		return eb_false;	/* already disconnected */
+		return false;	/* already disconnected */
 
 	stmt_text = "disconnect";
 	debugStatement();
 	rc = SQLDisconnect(hdbc);
 	if (errorTrap(0))
-		return eb_true;
+		return true;
 	clearAllCursors();	/* those handles are freed as well */
 	translevel = 0;
 	sql_database = 0;
-	return eb_false;
+	return false;
 }				/* disconnect */
 
 /* API level disconnect */
@@ -475,7 +475,7 @@ void sql_connect(const char *db, const char *login, const char *pw)
 		   4, &waste);
 	getdata_opts = 0;
 	SQLGetInfo(hdbc, SQL_GETDATA_EXTENSIONS, &getdata_opts, 4, &waste);
-	bookmarkBits = eb_false;
+	bookmarkBits = false;
 	SQLGetInfo(hdbc, SQL_BOOKMARK_PERSISTENCE, &bookmarkBits, 4, &waste);
 
 	exclist = 0;
@@ -495,7 +495,7 @@ void sql_connect(const char *db, const char *login, const char *pw)
 		current_driver = DRIVER_INFORMIX;
 	if (stringEqual(drivername, "libtdsodbc.so")) {
 		current_driver = DRIVER_TDS;
-		openfirst = eb_true;
+		openfirst = true;
 	}
 
 	if (sql_debug) {
@@ -538,7 +538,7 @@ void sql_begTrans(void)
 
 	/* count the nesting level of transactions. */
 	if (!translevel) {
-		badtrans = eb_false;
+		badtrans = false;
 		stmt_text = "begin work";
 		debugStatement();
 		rc = SQLSetConnectOption(hdbc, SQL_AUTOCOMMIT,
@@ -552,7 +552,7 @@ void sql_begTrans(void)
 }				/* sql_begTrans */
 
 /* end a transaction */
-static void endTrans(eb_bool commit)
+static void endTrans(bool commit)
 {
 	checkConnect();
 	stmt_text = 0;
@@ -576,7 +576,7 @@ static void endTrans(eb_bool commit)
 			errorTrap(0);
 		}
 	} else {		/* success or failure */
-		badtrans = eb_true;
+		badtrans = true;
 		if (!translevel) {	/* bottom level */
 			stmt_text = "rollback work";
 			debugStatement();
@@ -584,7 +584,7 @@ static void endTrans(eb_bool commit)
 			if (rc)
 				++translevel;
 			errorTrap(0);
-			badtrans = eb_false;
+			badtrans = false;
 		}
 	}			/* success or failure */
 
@@ -630,12 +630,12 @@ static void endTrans(eb_bool commit)
 
 void sql_commitWork(void)
 {
-	endTrans(eb_true);
+	endTrans(true);
 }
 
 void sql_rollbackWork(void)
 {
-	endTrans(eb_false);
+	endTrans(false);
 }
 
 void sql_deferConstraints(void)
@@ -677,7 +677,7 @@ sql_blobInsert(const char *tabname, const char *colname, int rowid,
 {
 	char blobcmd[100];
 	SQLINTEGER output_length;
-	eb_bool isfile;
+	bool isfile;
 	int fd;
 
 	/* basic sanity checks */
@@ -693,9 +693,9 @@ sql_blobInsert(const char *tabname, const char *colname, int rowid,
 	if (strlen(tabname) + strlen(colname) + 42 >= sizeof(blobcmd))
 		errorPrint("@internal blobInsert command too long");
 
-	isfile = eb_true;
+	isfile = true;
 	if (isnullstring(filename)) {
-		isfile = eb_false;
+		isfile = false;
 		if (!offset)
 			errorPrint
 			    ("2blobInsert is given null filename and null buffer");
@@ -704,7 +704,7 @@ sql_blobInsert(const char *tabname, const char *colname, int rowid,
 		fd = eopen(filename, O_RDONLY | O_BINARY, 0);
 		length = fileSizeByHandle(fd);
 		if (length == 0) {
-			isfile = eb_false;
+			isfile = false;
 			close(fd);
 		}
 	}
@@ -793,7 +793,7 @@ sql_blobInsert(const char *tabname, const char *colname, int rowid,
 	if (sql_debug)
 		appendFile(sql_debuglog, "%d rows affected", rv_lastNrows);
 	if (sql_debug2)
-		printf("%d rows affected\n", rv_lastNrows);
+		printf("%ld rows affected\n", rv_lastNrows);
 	SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
 	exclist = 0;
 }				/* sql_blobInsert */
@@ -810,14 +810,14 @@ static va_list sqlargs;
 
 /* Temp area to read the values as strings */
 static char retstring[NUMRETS][STRINGLEN + 4];
-static eb_bool everything_null;
+static bool everything_null;
 
 static void retsFromOdbc(void)
 {
 	void *q, *q1;
 	int i, l;
 	int fd, flags;
-	eb_bool yearfirst, indata = eb_false;
+	bool yearfirst, indata = false;
 	long dt;		/* temporarily hold date or time */
 	char *s;
 	short c_type;		/* C data type */
@@ -825,7 +825,7 @@ static void retsFromOdbc(void)
 	char tbuf[20];		/* temp buf, for dates and times */
 	double fmoney;		/* float version of money */
 	int blobcount = 0;
-	eb_bool fbc = fetchBlobColumns;
+	bool fbc = fetchBlobColumns;
 
 	/* no blobs unless proven otherwise */
 	rv_blobLoc = 0;
@@ -844,7 +844,7 @@ static void retsFromOdbc(void)
 				++blobcount;
 	if (blobcount > 1) {
 		i_puts(MSG_DBManyBlobs);
-		fbc = eb_false;
+		fbc = false;
 	}
 
 	for (i = 0; i < rv_numRets; ++i) {
@@ -853,7 +853,7 @@ static void retsFromOdbc(void)
 			if (!q) {
 				if (i)
 					break;
-				indata = eb_true;
+				indata = true;
 			}
 		}
 		if (indata) {
@@ -950,20 +950,20 @@ static void retsFromOdbc(void)
 		/* Postprocess the return values. */
 		/* For instance, turn string dates into our own 4-byte format. */
 		s = tbuf;
-		clipString(s);
+		trimWhite(s);
 		switch (rv_type[i]) {
 		case 'C':
 			*(char *)q = tbuf[0];
 			break;
 
 		case 'S':
-			clipString(q);
+			trimWhite(q);
 			break;
 
 		case 'D':
-			yearfirst = eb_false;
+			yearfirst = false;
 			if (s[4] == '-')
-				yearfirst = eb_true;
+				yearfirst = true;
 			dt = stringDate(s, yearfirst);
 			if (dt < 0)
 				errorPrint("@database holds invalid date %s",
@@ -1046,7 +1046,7 @@ static void retsFromOdbc(void)
 				    O_WRONLY | O_BINARY | O_CREAT | O_APPEND;
 			fd = eopen(rv_blobFile, flags, 0666);
 			rc = SQL_SUCCESS;
-			while (eb_true) {
+			while (true) {
 				int outbytes;
 				l = output_length;
 				if (l > sizeof(blobbuf))
@@ -1137,14 +1137,14 @@ available to the rest of the C routines in this file, and to the application.
 Returns false if the prepare failed.
 *********************************************************************/
 
-static eb_bool prepareInternal(const char *stmt)
+static bool prepareInternal(const char *stmt)
 {
 	short i, nc, coltype, colscale, nullable, namelen;
 	unsigned long colprec;
-	eb_bool blobpresent = eb_false;
+	bool blobpresent = false;
 
 	checkConnect();
-	everything_null = eb_true;
+	everything_null = true;
 	if (isnullstring(stmt))
 		errorPrint("2null SQL statement");
 	stmt_text = stmt;
@@ -1157,7 +1157,7 @@ static eb_bool prepareInternal(const char *stmt)
 		if (!strstr(stmt, "where") && !strstr(stmt, "WHERE")) {
 			showStatement();
 			setError(MSG_DBNoWhere);
-			return eb_false;
+			return false;
 		}
 
 	rv_numRets = 0;
@@ -1170,12 +1170,12 @@ static eb_bool prepareInternal(const char *stmt)
 	else
 		rc = SQLPrepare(hstmt, (char *)stmt, SQL_NTS);
 	if (errorTrap(0))
-		return eb_false;
+		return false;
 
 	/* gather column headings and types */
 	rc = SQLNumResultCols(hstmt, &nc);
 	if (errorTrap(0))
-		return eb_false;
+		return false;
 
 	if (nc > NUMRETS) {
 		showStatement();
@@ -1187,7 +1187,7 @@ static eb_bool prepareInternal(const char *stmt)
 				    rv_name[i], COLNAMELEN, &namelen,
 				    &coltype, &colprec, &colscale, &nullable);
 		if (errorTrap("01004"))
-			return eb_false;
+			return false;
 
 /*********************************************************************
 The following is an Informix kludge,
@@ -1270,7 +1270,7 @@ Count(*) becomes decimal(15,0).  So be careful.
 				errorPrint
 				    ("2Statement selects more than one blob column");
 			}
-			blobpresent = eb_true;
+			blobpresent = true;
 			rv_type[i] = (coltype == SQL_LONGVARCHAR ? 'T' : 'B');
 			break;
 
@@ -1284,7 +1284,7 @@ Count(*) becomes decimal(15,0).  So be careful.
 	}			/* loop over returns */
 
 	rv_numRets = nc;
-	return eb_true;
+	return true;
 }				/* prepareInternal */
 
 /*********************************************************************
@@ -1298,20 +1298,20 @@ A mode variable says whether execution or selection or both are allowed.
 Return true if data was successfully fetched.
 *********************************************************************/
 
-static eb_bool execInternal(const char *stmt, int mode)
+static bool execInternal(const char *stmt, int mode)
 {
-	eb_bool notfound = eb_false;
+	bool notfound = false;
 
 	newStatement();
 	if (!prepareInternal(stmt))
-		return eb_false;	/* error */
+		return false;	/* error */
 
 	if (!rv_numRets) {
 		if (!(mode & 1)) {
 			showStatement();
 			errorPrint("2SQL select statement returns no values");
 		}
-		notfound = eb_true;
+		notfound = true;
 	} else {		/* end no return values */
 		if (!(mode & 2)) {
 			showStatement();
@@ -1332,24 +1332,24 @@ static eb_bool execInternal(const char *stmt, int mode)
 					    1);
 			if (rc == SQL_NO_DATA) {
 				rc = SQL_SUCCESS;
-				notfound = eb_true;
+				notfound = true;
 			} else
-				everything_null = eb_false;
+				everything_null = false;
 		} else {
 			rc = SQLRowCount(hstmt, &rv_lastNrows);
 			if (sql_debug)
 				appendFile(sql_debuglog, "%d rows affected",
 					   rv_lastNrows);
 			if (sql_debug2)
-				printf("%d rows affected\n", rv_lastNrows);
+				printf("%ld rows affected\n", rv_lastNrows);
 		}
 	}
 
 	if (errorTrap(0))
-		return eb_false;
+		return false;
 
 	if (!rv_numRets)
-		return eb_true;
+		return true;
 	return !notfound;
 }				/* execInternal */
 
@@ -1358,18 +1358,18 @@ Run individual select or execute statements, using the above internal routine.
 *********************************************************************/
 
 /* execute a stand-alone statement with no % formatting of the string */
-eb_bool sql_execNF(const char *stmt)
+bool sql_execNF(const char *stmt)
 {
-	eb_bool ok = execInternal(stmt, 1);
+	bool ok = execInternal(stmt, 1);
 	SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
 	exclist = 0;
 	return ok;
 }				/* sql_execNF */
 
 /* execute a stand-alone statement with % formatting */
-eb_bool sql_exec(const char *stmt, ...)
+bool sql_exec(const char *stmt, ...)
 {
-	eb_bool ok;
+	bool ok;
 	va_start(sqlargs, stmt);
 	stmt = lineFormatStack(stmt, 0, &sqlargs);
 	ok = execInternal(stmt, 1);
@@ -1381,9 +1381,9 @@ eb_bool sql_exec(const char *stmt, ...)
 
 /* run a select statement with % formatting */
 /* return true if the row was found */
-eb_bool sql_select(const char *stmt, ...)
+bool sql_select(const char *stmt, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, stmt);
 	stmt = lineFormatStack(stmt, 0, &sqlargs);
 	rowfound = execInternal(stmt, 2);
@@ -1393,9 +1393,9 @@ eb_bool sql_select(const char *stmt, ...)
 }				/* sql_select */
 
 /* run a select statement with no % formatting of the string */
-eb_bool sql_selectNF(const char *stmt, ...)
+bool sql_selectNF(const char *stmt, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, stmt);
 	rowfound = execInternal(stmt, 2);
 	retsFromOdbc();
@@ -1406,7 +1406,7 @@ eb_bool sql_selectNF(const char *stmt, ...)
 /* run a select statement with one return value */
 int sql_selectOne(const char *stmt, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, stmt);
 	stmt = lineFormatStack(stmt, 0, &sqlargs);
 	rowfound = execInternal(stmt, 2);
@@ -1420,9 +1420,9 @@ int sql_selectOne(const char *stmt, ...)
 }				/* sql_selectOne */
 
 /* run a stored procedure with no % formatting */
-static eb_bool sql_procGo(const char *stmt)
+static bool sql_procGo(const char *stmt)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	char *s = allocMem(20 + strlen(stmt));
 	strcpy(s, "execute procedure ");
 	strcat(s, stmt);
@@ -1433,9 +1433,9 @@ static eb_bool sql_procGo(const char *stmt)
 }				/* sql_procGo */
 
 /* run a stored procedure */
-eb_bool sql_proc(const char *stmt, ...)
+bool sql_proc(const char *stmt, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, stmt);
 	stmt = lineFormatStack(stmt, 0, &sqlargs);
 	rowfound = sql_procGo(stmt);
@@ -1447,7 +1447,7 @@ eb_bool sql_proc(const char *stmt, ...)
 /* run a stored procedure with one return */
 int sql_procOne(const char *stmt, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, stmt);
 	stmt = lineFormatStack(stmt, 0, &sqlargs);
 	rowfound = sql_procGo(stmt);
@@ -1465,7 +1465,7 @@ Prepare, open, close, and free SQL cursors.
 *********************************************************************/
 
 /* prepare a cursor; return the ID number of that cursor */
-static int prepareCursor(const char *stmt, eb_bool scrollflag)
+static int prepareCursor(const char *stmt, bool scrollflag)
 {
 	struct OCURS *o = findNewCursor();
 	stmt = lineFormatStack(stmt, 0, &sqlargs);
@@ -1495,7 +1495,7 @@ int sql_prepare(const char *stmt, ...)
 {
 	int n;
 	va_start(sqlargs, stmt);
-	n = prepareCursor(stmt, eb_false);
+	n = prepareCursor(stmt, false);
 	exclist = 0;
 	if (n < 0)
 		SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
@@ -1506,7 +1506,7 @@ int sql_prepareScrolling(const char *stmt, ...)
 {
 	int n;
 	va_start(sqlargs, stmt);
-	n = prepareCursor(stmt, eb_true);
+	n = prepareCursor(stmt, true);
 	exclist = 0;
 	if (n < 0)
 		SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
@@ -1539,7 +1539,7 @@ int sql_prepOpen(const char *stmt, ...)
 	struct OCURS *o;
 
 	va_start(sqlargs, stmt);
-	n = prepareCursor(stmt, eb_false);
+	n = prepareCursor(stmt, false);
 	if (n < 0) {
 		SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
 		return n;
@@ -1611,19 +1611,19 @@ void sql_closeFree(int cid)
 
 /* fetch row n from the open cursor.
  * Flag can be used to fetch first, last, next, or previous. */
-static eb_bool fetchInternal(int cid, long n, int flag)
+static bool fetchInternal(int cid, long n, int flag)
 {
 	long nextrow, lastrow;
 	struct OCURS *o = findCursor(cid);
 
-	everything_null = eb_true;
+	everything_null = true;
 
 	/* don't do the fetch if we're looking for row 0 absolute,
 	 * that just nulls out the return values */
 	if (flag == SQL_FD_FETCH_ABSOLUTE && !n) {
 		o->rownum = 0;
 fetchZero:
-		return eb_false;
+		return false;
 	}
 
 	lastrow = nextrow = o->rownum;
@@ -1670,53 +1670,53 @@ fetchZero:
 	hstmt = o->hstmt;
 	rc = SQLFetchScroll(hstmt, (ushort) flag, nextrow);
 	if (rc == SQL_NO_DATA)
-		return eb_false;
+		return false;
 	if (errorTrap(0))
-		return eb_false;
+		return false;
 	o->rownum = nextrow;
-	everything_null = eb_false;
-	return eb_true;
+	everything_null = false;
+	return true;
 }				/* fetchInternal */
 
-eb_bool sql_fetchFirst(int cid, ...)
+bool sql_fetchFirst(int cid, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, cid);
 	rowfound = fetchInternal(cid, 0L, SQL_FD_FETCH_FIRST);
 	retsFromOdbc();
 	return rowfound;
 }				/* sql_fetchFirst */
 
-eb_bool sql_fetchLast(int cid, ...)
+bool sql_fetchLast(int cid, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, cid);
 	rowfound = fetchInternal(cid, 0L, SQL_FD_FETCH_LAST);
 	retsFromOdbc();
 	return rowfound;
 }				/* sql_fetchLast */
 
-eb_bool sql_fetchNext(int cid, ...)
+bool sql_fetchNext(int cid, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, cid);
 	rowfound = fetchInternal(cid, 0L, SQL_FD_FETCH_NEXT);
 	retsFromOdbc();
 	return rowfound;
 }				/* sql_fetchNext */
 
-eb_bool sql_fetchPrev(int cid, ...)
+bool sql_fetchPrev(int cid, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, cid);
 	rowfound = fetchInternal(cid, 0L, SQL_FD_FETCH_PREV);
 	retsFromOdbc();
 	return rowfound;
 }				/* sql_fetchPrev */
 
-eb_bool sql_fetchAbs(int cid, long rownum, ...)
+bool sql_fetchAbs(int cid, long rownum, ...)
 {
-	eb_bool rowfound;
+	bool rowfound;
 	va_start(sqlargs, rownum);
 	rowfound = fetchInternal(cid, rownum, SQL_FD_FETCH_ABSOLUTE);
 	retsFromOdbc();
@@ -1790,7 +1790,7 @@ done:
 	return;
 }				/* getPrimaryKey */
 
-eb_bool showTables(void)
+bool showTables(void)
 {
 	char tabname[40];
 	char tabtype[40];
@@ -1828,19 +1828,19 @@ SQLSetConnectAttr(hdbc, SQL_ATTR_METADATA_ID,
 		stringAndString(&buf, &buflen, tabline);
 	}
 
-	cx = sideBuffer(0, buf, buflen, 0, eb_false);
+	cx = sideBuffer(0, buf, buflen, 0);
 	nzFree(buf);
 	i_printf(MSG_ShowTables, cx);
 	SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
-	return eb_true;
+	return true;
 
 abort:
 	SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
-	return eb_false;
+	return false;
 }				/* showTables */
 
 /* display foreign keys, from this table to others */
-eb_bool fetchForeign(char *tname)
+bool fetchForeign(char *tname)
 {
 	char farschema[40], fartab[40];
 	char farcol[40];
@@ -1883,9 +1883,9 @@ eb_bool fetchForeign(char *tname)
 	}
 
 	SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
-	return eb_true;
+	return true;
 
 abort:
 	SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
-	return eb_false;
+	return false;
 }				/* fetchForeign */
