@@ -1,29 +1,27 @@
 /* main.c
  * Entry point, arguments and options.
- * Copyright (c) Karl Dahlke, 2008
  * This file is part of the edbrowse project, released under GPL.
  */
 
 #include "eb.h"
 
 #include <sys/stat.h>
-#include <signal.h>
 #include <pcre.h>
 
 /* Define the globals that are declared in eb.h. */
 /* See eb.h for descriptive comments. */
 
-const char *version = "3.5.1";
+const char *version = "3.6.0.1";
+const char *progname;
 char *userAgents[10], *currentAgent, *currentReferrer;
 const char eol[] = "\r\n";
-char EMPTYSTRING[] = "";
-int debugLevel = 1;
+int jsPool = 32;
+int webTimeout = 20, mailTimeout = 0;
 char *sslCerts;
 int verifyCertificates = 1;
-int webTimeout = 20, mailTimeout = 0;
-eb_bool ismc, browseLocal, passMail, errorExit;
-eb_bool isInteractive, inInput, listNA;
-volatile eb_bool intFlag;
+bool ismc, isimap, passMail;
+bool inInput, listNA;
+volatile bool intFlag;
 int fileSize;
 int localAccount, maxAccount;
 struct MACCOUNT accounts[MAXACCOUNT];
@@ -34,22 +32,18 @@ struct PXENT proxyEntries[MAXPROXY];
 static int maxTables;
 static struct DBTABLE dbtables[MAXDBT];
 char *dbarea, *dblogin, *dbpw;	/* to log into the database */
-eb_bool fetchBlobColumns;
-eb_bool caseInsensitive, searchStringsAll;
-eb_bool allowRedirection = eb_true, allowJS = eb_true, sendReferrer = eb_true;
-eb_bool binaryDetect = eb_true;
-eb_bool inputReadLine;
-eb_bool showHiddenFiles, helpMessagesOn;
-uchar dirWrite, endMarks;
+bool fetchBlobColumns;
+bool caseInsensitive, searchStringsAll;
+bool allowRedirection = true, allowJS = true, sendReferrer = true;
+bool binaryDetect = true, pluginsOn = true;
+bool inputReadLine;
 int context = 1;
-uchar linePending[MAXTTYLINE];
+pst linePending;
 char *changeFileName, *mailDir;
-char *mailUnread, *mailStash;
-char *addressFile, *ipbFile;
-char *home, *recycleBin, *configFile, *sigFile, *sigFileEnd;
-char *cookieFile, *spamCan;
-char *edbrowseTempFile, *edbrowseTempPDF, *edbrowseTempHTML;
-struct ebWindow *cw;
+char *mailUnread, *mailStash, *mailReply;
+char *addressFile;
+char *recycleBin, *configFile, *sigFile, *sigFileEnd;
+char *cookieFile;
 struct ebSession sessionList[MAXSESSION], *cs;
 
 /* Edbrowse functions, defined in the config file */
@@ -74,6 +68,9 @@ static char *cfgcopy;
 static int cfglen;
 static long nowday;
 
+#ifdef _MSC_VER
+#endif // _MSC_VER
+
 static void setNowDay(void)
 {
 	time_t now;
@@ -84,71 +81,16 @@ static void setNowDay(void)
 	nowday = now;
 }				/* setNowDay */
 
-static void updateConfig(void)
-{
-	int fh = open(configFile, O_WRONLY | O_TRUNC, 0);
-	if (fh < 0) {
-		i_puts(MSG_ConfigUpdate);
-		return;
-	}
-	if (write(fh, cfgcopy, cfglen) < cfglen)
-		i_printfExit(MSG_ERBC_NoWrite);
-	close(fh);
-}				/* updateConfig */
-
-eb_bool junkSubject(const char *s, char key)
-{
-	int l, n;
-	char *new;
-	long exp = nowday;
-	if (!s || !*s) {
-		i_puts(MSG_NoSubject);
-		return eb_false;
-	}
-	if (!cfgcopy) {
-		i_puts(MSG_NoConfig);
-		return eb_false;
-	}
-	if (!subjstart) {
-		i_puts(MSG_NoSubjFilter);
-		return eb_false;
-	}
-	if (key == 'j')
-		exp += 10;
-	if (key == 'J')
-		exp += 365;
-	l = strlen(s) + 10;
-	if (exp > 9999)
-		++l;
-	n = cfglen + l;
-	new = allocMem(n);
-	memcpy(new, cfgcopy, subjstart);
-	sprintf(new + subjstart, "%ld`%s > x\n", exp, s);
-	memcpy(new + subjstart + l, cfgcopy + subjstart, cfglen - subjstart);
-	nzFree(cfgcopy);
-	cfgcopy = new;
-	cfglen = n;
-	updateConfig();
-	if (n_filters < MAXFILTER - 1) {
-		filters[n_filters].type = 4;
-		filters[n_filters].match = cloneString(s);
-		filters[n_filters].redirect = "x";
-		++n_filters;
-	}
-	return eb_true;
-}				/* junkSubject */
-
 /* This routine succeeds, or aborts via i_printfExit */
 static void readConfigFile(void)
 {
 	char *buf, *s, *t, *v, *q;
-	char *cfglp, *cfgnlp;
 	int buflen, n;
 	char c, ftype;
-	eb_bool cmt = eb_false;
-	eb_bool startline = eb_true;
-	eb_bool cfgmodify = eb_false;
-	uchar mailblock = 0, mimeblock = 0, tabblock = 0;
+	bool cmt = false;
+	bool startline = true;
+	uchar mailblock = 0;
+	bool mimeblock = false, tabblock = false;
 	int nest, ln, j;
 	int sn = 0;		/* script number */
 	char stack[MAXNEST];
@@ -158,28 +100,32 @@ static void readConfigFile(void)
 	struct PXENT *px;
 	struct MIMETYPE *mt;
 	struct DBTABLE *td;
+
+/* Order is important here: mail{}, mime{}, table{}, then global keywords */
+#define MAILWORDS 0
+#define MIMEWORDS 8
+#define TABLEWORDS 15
+#define GLOBALWORDS 19
 	static const char *const keywords[] = {
 		"inserver", "outserver", "login", "password", "from", "reply",
 		"inport", "outport",
 		"type", "desc", "suffix", "protocol", "program",
+		"content", "outtype",
 		"tname", "tshort", "cols", "keycol",
-		"adbook", "ipblack", "maildir", "agent",
-		"jar", "nojs", "spamcan",
+		"adbook", "downdir", "maildir", "agent",
+		"jar", "nojs", "xyz@xyz",
 		"webtimer", "mailtimer", "certfile", "datasource", "proxy",
 		"linelength", "localizeweb", "jspool", "novs",
 		0
 	};
 
-	if (!fileTypeByName(configFile, eb_false))
+	if (!fileTypeByName(configFile, false))
 		return;		/* config file not present */
 	if (!fileIntoMemory(configFile, &buf, &buflen))
 		showErrorAbort();
 /* An extra newline won't hurt. */
 	if (buflen && buf[buflen - 1] != '\n')
 		buf[buflen++] = '\n';
-/* make copy */
-	cfgcopy = allocMem(buflen + 1);
-	memcpy(cfgcopy, buf, (cfglen = buflen));
 
 /* Undos, uncomment, watch for nulls */
 /* Encode mail{ as hex 81 m, and other encodings. */
@@ -187,18 +133,21 @@ static void readConfigFile(void)
 	for (s = t = v = buf; s < buf + buflen; ++s) {
 		c = *s;
 		if (c == '\0')
-			i_printfExit(MSG_ERBC_Nulls, ln);
+			i_printfExit(MSG_EBRC_Nulls, ln);
 		if (c == '\r' && s[1] == '\n')
 			continue;
+
 		if (cmt) {
 			if (c != '\n')
 				continue;
-			cmt = eb_false;
+			cmt = false;
 		}
+
 		if (c == '#' && startline) {
-			cmt = eb_true;
+			cmt = true;
 			goto putc;
 		}
+
 		if (c == '\n') {
 			last[lidx] = 0;
 			lidx = 0;
@@ -215,7 +164,8 @@ static void readConfigFile(void)
 				v[1] = 'm';
 				t = v + 2;
 			}
-			if (stringEqual(last, "mime{")) {
+			if (stringEqual(last, "plugin{") ||
+			    stringEqual(last, "mime{")) {
 				*v = '\x81';
 				v[1] = 'e';
 				t = v + 2;
@@ -239,8 +189,6 @@ static void readConfigFile(void)
 				*v = '\x81';
 				v[1] = 's';
 				t = v + 2;
-				if (!subjstart)
-					subjstart = s + 1 - buf;
 			}
 			if (stringEqual(last, "if(*){")) {
 				*v = '\x81';
@@ -288,34 +236,37 @@ static void readConfigFile(void)
 			    (last[8] == '+' || last[8] == ':')) {
 				q = last + 9;
 				if (*q == 0 || *q == '{' || *q == '(')
-					i_printfExit(MSG_ERBC_NoFnName, ln);
+					i_printfExit(MSG_EBRC_NoFnName, ln);
 				if (isdigitByte(*q))
-					i_printfExit(MSG_ERBC_FnDigit, ln);
+					i_printfExit(MSG_EBRC_FnDigit, ln);
 				while (isalnumByte(*q))
 					++q;
 				if (q - last - 9 > 10)
-					i_printfExit(MSG_ERBC_FnTooLong, ln);
+					i_printfExit(MSG_EBRC_FnTooLong, ln);
 				if (*q != '{' || q[1])
-					i_printfExit(MSG_ERBC_SyntaxErr, ln);
+					i_printfExit(MSG_EBRC_SyntaxErr, ln);
 				last[7] = 'f';
 				last[6] = '\x81';
 				strcpy(v, last + 6);
 				t = v + strlen(v);
 			}
+
 			*t++ = c;
 			v = t;
 			++ln;
-			startline = eb_true;
+			startline = true;
 			continue;
 		}
+
 		if (c == ' ' || c == '\t') {
 			if (startline)
 				continue;
 		} else {
 			if (lidx < sizeof(last) - 1)
 				last[lidx++] = c;
-			startline = eb_false;
+			startline = false;
 		}
+
 putc:
 		*t++ = c;
 	}
@@ -326,8 +277,7 @@ putc:
 	nest = 0;
 	stack[0] = ' ';
 
-	for (s = buf, cfglp = cfgcopy; *s; s = t + 1, cfglp = cfgnlp, ++ln) {
-		cfgnlp = strchr(cfglp, '\n') + 1;
+	for (s = buf; *s; s = t + 1, ++ln) {
 		t = strchr(s, '\n');
 		if (t == s)
 			continue;	/* empty line */
@@ -339,11 +289,11 @@ putc:
 		if (mailblock > 1 && !strchr("\x81\x82\x83", *s)) {
 			v = strchr(s, '>');
 			if (!v)
-				i_printfExit(MSG_ERBC_NoCondFile, ln);
+				i_printfExit(MSG_EBRC_NoCondFile, ln);
 			while (v > s && (v[-1] == ' ' || v[-1] == '\t'))
 				--v;
 			if (v == s)
-				i_printfExit(MSG_ERBC_NoMatchStr, ln);
+				i_printfExit(MSG_EBRC_NoMatchStr, ln);
 			c = *v, *v++ = 0;
 			if (c != '>') {
 				while (*v != '>')
@@ -353,27 +303,11 @@ putc:
 			while (*v == ' ' || *v == '\t')
 				++v;
 			if (!*v)
-				i_printfExit(MSG_ERBC_MatchNowh, ln, s);
+				i_printfExit(MSG_EBRC_MatchNowh, ln, s);
 			if (n_filters == MAXFILTER - 1)
-				i_printfExit(MSG_ERBC_Filters, ln);
-			filters[n_filters].redirect = v;
-			if (mailblock >= 2) {
-				long exp = strtol(s, &v, 10);
-				if (exp > 0 && *v == '`' && v[1]) {
-					s = v + 1;
-					filters[n_filters].expire = exp;
-					if (exp <= nowday) {
-						cfgmodify = eb_true;
-						memmove(cfglp, cfgnlp,
-							cfgcopy + cfglen -
-							cfgnlp);
-						cfglen -= (cfgnlp - cfglp);
-						cfgnlp = cfglp;
-						continue;
-					}	/* filter rule out of date */
-				}
-			}
+				i_printfExit(MSG_EBRC_Filters, ln);
 			filters[n_filters].match = s;
+			filters[n_filters].redirect = v;
 			filters[n_filters].type = mailblock;
 			++n_filters;
 			continue;
@@ -397,28 +331,31 @@ putc:
 		n = stringInList(keywords, s);
 		if (n < 0) {
 			if (!nest)
-				i_printfExit(MSG_ERBC_BadKeyword, s, ln);
+				i_printfExit(MSG_EBRC_BadKeyword, s, ln);
 			*v = c;	/* put it back */
 			goto nokeyword;
 		}
 
-		if (n < 8 && mailblock != 1)
-			i_printfExit(MSG_ERBC_MailAttrOut, ln, s);
+		if (nest)
+			i_printfExit(MSG_EBRC_KeyInFunc, ln);
 
-		if (n >= 8 && n < 13 && mimeblock != 1)
-			i_printfExit(MSG_ERBC_MimeAttrOut, ln, s);
+		if (n < MIMEWORDS && mailblock != 1)
+			i_printfExit(MSG_EBRC_MailAttrOut, ln, s);
 
-		if (n >= 13 && n < 17 && tabblock != 1)
-			i_printfExit(MSG_ERBC_TableAttrOut, ln, s);
+		if (n >= MIMEWORDS && n < TABLEWORDS && !mimeblock)
+			i_printfExit(MSG_EBRC_MimeAttrOut, ln, s);
 
-		if (n >= 8 && mailblock)
-			i_printfExit(MSG_ERBC_MailAttrIn, ln, s);
+		if (n >= TABLEWORDS && n < GLOBALWORDS && !tabblock)
+			i_printfExit(MSG_EBRC_TableAttrOut, ln, s);
 
-		if ((n < 8 || n >= 13) && mimeblock)
-			i_printfExit(MSG_ERBC_MimeAttrIn, ln, s);
+		if (n >= MIMEWORDS && mailblock)
+			i_printfExit(MSG_EBRC_MailAttrIn, ln, s);
 
-		if ((n < 13 || n >= 17) && tabblock)
-			i_printfExit(MSG_ERBC_TableAttrIn, ln, s);
+		if ((n < MIMEWORDS || n >= TABLEWORDS) && mimeblock)
+			i_printfExit(MSG_EBRC_MimeAttrIn, ln, s);
+
+		if ((n < TABLEWORDS || n >= GLOBALWORDS) && tabblock)
+			i_printfExit(MSG_EBRC_TableAttrIn, ln, s);
 
 /* act upon the keywords */
 		++v;
@@ -430,40 +367,40 @@ putc:
 		while (*v == ' ' || *v == '\t')
 			++v;
 		if (!*v)
-			i_printfExit(MSG_ERBC_NoAttr, ln, s);
+			i_printfExit(MSG_EBRC_NoAttr, ln, s);
 
 		switch (n) {
-		case 0:
+		case 0:	/* inserver */
 			act->inurl = v;
 			continue;
 
-		case 1:
+		case 1:	/* outserver */
 			act->outurl = v;
 			continue;
 
-		case 2:
+		case 2:	/* login */
 			act->login = v;
 			continue;
 
-		case 3:
+		case 3:	/* password */
 			act->password = v;
 			continue;
 
-		case 4:
+		case 4:	/* from */
 			act->from = v;
 			continue;
 
-		case 5:
+		case 5:	/* reply */
 			act->reply = v;
 			continue;
 
-		case 6:
+		case 6:	/* inport */
 			if (*v == '*')
 				act->inssl = 1, ++v;
 			act->inport = atoi(v);
 			continue;
 
-		case 7:
+		case 7:	/* outport */
 			if (*v == '+')
 				act->outssl = 4, ++v;
 			if (*v == '^')
@@ -473,40 +410,50 @@ putc:
 			act->outport = atoi(v);
 			continue;
 
-		case 8:
-			if (*v == '<')
-				mt->stream = eb_true, ++v;
+		case 8:	/* type */
 			mt->type = v;
 			continue;
 
-		case 9:
+		case 9:	/* desc */
 			mt->desc = v;
 			continue;
 
-		case 10:
+		case 10:	/* suffix */
 			mt->suffix = v;
 			continue;
 
-		case 11:
+		case 11:	/* protocol */
 			mt->prot = v;
+			mt->stream = true;
 			continue;
 
-		case 12:
+		case 12:	/* program */
 			mt->program = v;
 			continue;
 
-		case 13:
+		case 13:	/* content */
+			mt->content = v;
+			continue;
+
+		case 14:	/* outtype */
+			c = tolower(*v);
+			if (c != 'h' && c != 't')
+				i_printfExit(MSG_EBRC_Outtype, ln);
+			mt->outtype = c;
+			continue;
+
+		case 15:	/* tname */
 			td->name = v;
 			continue;
 
-		case 14:
+		case 16:	/* tshort */
 			td->shortname = v;
 			continue;
 
-		case 15:
+		case 17:	/* cols */
 			while (*v) {
 				if (td->ncols == MAXTCOLS)
-					i_printfExit(MSG_ERBC_ManyCols, ln,
+					i_printfExit(MSG_EBRC_ManyCols, ln,
 						     MAXTCOLS);
 				td->cols[td->ncols++] = v;
 				q = strchr(v, ',');
@@ -517,113 +464,107 @@ putc:
 			}
 			continue;
 
-		case 16:
+		case 18:	/* keycol */
 			if (!isdigitByte(*v))
-				i_printfExit(MSG_ERBC_KeyNotNb, ln);
+				i_printfExit(MSG_EBRC_KeyNotNb, ln);
 			td->key1 = strtol(v, &v, 10);
 			if (*v == ',' && isdigitByte(v[1]))
 				td->key2 = strtol(v + 1, &v, 10);
 			if (td->key1 > td->ncols || td->key2 > td->ncols)
-				i_printfExit(MSG_ERBC_KeyOutRange, ln,
+				i_printfExit(MSG_EBRC_KeyOutRange, ln,
 					     td->ncols);
 			continue;
 
-		case 17:
+		case 19:	/* adbook */
 			addressFile = v;
-			ftype = fileTypeByName(v, eb_false);
+			ftype = fileTypeByName(v, false);
 			if (ftype && ftype != 'f')
-				i_printfExit(MSG_ERBC_AbNotFile, v);
+				i_printfExit(MSG_EBRC_AbNotFile, v);
 			continue;
 
-		case 18:
-			ipbFile = v;
-			ftype = fileTypeByName(v, eb_false);
-			if (ftype && ftype != 'f')
-				i_printfExit(MSG_ERBC_IPNotFile, v);
+		case 20:	/* downdir */
+			downDir = v;
+			if (fileTypeByName(v, false) != 'd')
+				i_printfExit(MSG_EBRC_NotDir, v);
 			continue;
 
-		case 19:
+		case 21:	/* maildir */
 			mailDir = v;
-			if (fileTypeByName(v, eb_false) != 'd')
-				i_printfExit(MSG_ERBC_NotDir, v);
-			mailUnread = allocMem(strlen(v) + 12);
+			if (fileTypeByName(v, false) != 'd')
+				i_printfExit(MSG_EBRC_NotDir, v);
+			mailUnread = allocMem(strlen(v) + 20);
 			sprintf(mailUnread, "%s/unread", v);
 /* We need the unread directory, else we can't fetch mail. */
 /* Create it if it isn't there. */
-			if (fileTypeByName(mailUnread, eb_false) != 'd') {
+			if (fileTypeByName(mailUnread, false) != 'd') {
 				if (mkdir(mailUnread, 0700))
-					i_printfExit(MSG_ERBC_NotDir,
+					i_printfExit(MSG_EBRC_NotDir,
 						     mailUnread);
 			}
+			mailReply = allocMem(strlen(v) + 20);
+			sprintf(mailReply, "%s/.reply", v);
 			continue;
 
-		case 20:
+		case 22:	/* agent */
 			for (j = 0; j < 10; ++j)
 				if (!userAgents[j])
 					break;
 			if (j == 10)
-				i_printfExit(MSG_ERBC_ManyAgents, ln);
+				i_printfExit(MSG_EBRC_ManyAgents, ln);
 			userAgents[j] = v;
 			continue;
 
-		case 21:
+		case 23:	/* jar */
 			cookieFile = v;
-			ftype = fileTypeByName(v, eb_false);
+			ftype = fileTypeByName(v, false);
 			if (ftype && ftype != 'f')
-				i_printfExit(MSG_ERBC_JarNotFile, v);
+				i_printfExit(MSG_EBRC_JarNotFile, v);
 			j = open(v, O_WRONLY | O_APPEND | O_CREAT, 0600);
 			if (j < 0)
-				i_printfExit(MSG_ERBC_JarNoWrite, v);
+				i_printfExit(MSG_EBRC_JarNoWrite, v);
 			close(j);
 			continue;
 
-		case 22:
+		case 24:	/* nojs */
 			if (javaDisCount == MAXNOJS)
-				i_printfExit(MSG_ERBC_NoJS, MAXNOJS);
+				i_printfExit(MSG_EBRC_NoJS, MAXNOJS);
 			if (*v == '.')
 				++v;
 			q = strchr(v, '.');
 			if (!q || q[1] == 0)
-				i_printfExit(MSG_ERBC_DomainDot, ln, v);
+				i_printfExit(MSG_EBRC_DomainDot, ln, v);
 			javaDis[javaDisCount++] = v;
 			continue;
 
-		case 23:
-			spamCan = v;
-			ftype = fileTypeByName(v, eb_false);
-			if (ftype && ftype != 'f')
-				i_printfExit(MSG_ERBC_TrashNotFile, v);
-			continue;
-
-		case 24:
+		case 26:	/* webtimer */
 			webTimeout = atoi(v);
 			continue;
 
-		case 25:
+		case 27:	/* mailtimer */
 			mailTimeout = atoi(v);
 			continue;
 
-		case 26:
+		case 28:	/* certfile */
 			sslCerts = v;
-			ftype = fileTypeByName(v, eb_false);
+			ftype = fileTypeByName(v, false);
 			if (ftype && ftype != 'f')
-				i_printfExit(MSG_ERBC_SSLNoFile, v);
+				i_printfExit(MSG_EBRC_SSLNoFile, v);
 			j = open(v, O_RDONLY);
 			if (j < 0)
-				i_printfExit(MSG_ERBC_SSLNoRead, v);
+				i_printfExit(MSG_EBRC_SSLNoRead, v);
 			close(j);
 			continue;
 
-		case 27:
+		case 29:	/* datasource */
 			setDataSource(v);
 			continue;
 
-		case 28:	/* proxy */
+		case 30:	/* proxy */
 			if (maxproxy == MAXPROXY)
-				i_printfExit(MSG_ERBC_NoPROXY, MAXPROXY);
+				i_printfExit(MSG_EBRC_NoPROXY, MAXPROXY);
 			px = proxyEntries + maxproxy;
 			maxproxy++;
-			spaceCrunch(v, eb_true, eb_true);
+			spaceCrunch(v, true, true);
 			q = strchr(v, ' ');
 			if (q) {
 				*q = 0;
@@ -642,19 +583,19 @@ putc:
 				px->proxy = v;
 			continue;
 
-		case 29:	/* linelength */
+		case 31:	/* linelength */
 			displayLength = atoi(v);
 			if (displayLength < 80)
 				displayLength = 80;
 			continue;
 
-		case 30:	/* localizeweb */
+		case 32:	/* localizeweb */
 /* We should probably allow autodetection of language. */
 /* E.G., the keyword auto indicates that you want autodetection. */
 			setHTTPLanguage(v);
 			continue;
 
-		case 31:	/* jspool */
+		case 33:	/* jspool */
 			jsPool = atoi(v);
 			if (jsPool < 2)
 				jsPool = 2;
@@ -662,17 +603,17 @@ putc:
 				jsPool = 1000;
 			continue;
 
-		case 32:
+		case 34:	/* novs */
 			if (*v == '.')
 				++v;
 			q = strchr(v, '.');
 			if (!q || q[1] == 0)
-				i_printfExit(MSG_ERBC_DomainDot, ln, v);
+				i_printfExit(MSG_EBRC_DomainDot, ln, v);
 			addNovsHost(v);
 			continue;
 
 		default:
-			i_printfExit(MSG_ERBC_KeywordNYI, ln, s);
+			i_printfExit(MSG_EBRC_KeywordNYI, ln, s);
 		}		/* switch */
 
 nokeyword:
@@ -681,13 +622,32 @@ nokeyword:
 			if (localAccount == maxAccount + 1)
 				continue;
 			if (localAccount)
-				i_printfExit(MSG_ERBC_SevDefaults);
+				i_printfExit(MSG_EBRC_SevDefaults);
 			localAccount = maxAccount + 1;
 			continue;
 		}
 
 		if (stringEqual(s, "nofetch") && mailblock == 1) {
-			act->nofetch = 1;
+			act->nofetch = true;
+			continue;
+		}
+
+		if (stringEqual(s, "secure") && mailblock == 1) {
+			act->secure = true;
+			continue;
+		}
+
+		if (stringEqual(s, "imap") && mailblock == 1) {
+			act->imap = act->nofetch = true;
+			continue;
+		}
+
+		if (stringEqual(s, "download") && mimeblock == 1) {
+			mt->download = true;
+			continue;
+		}
+		if (stringEqual(s, "stream") && mimeblock == 1) {
+			mt->stream = true;
 			continue;
 		}
 
@@ -696,21 +656,28 @@ nokeyword:
 				++maxAccount;
 				mailblock = 0;
 				if (!act->inurl)
-					i_printfExit(MSG_ERBC_NoInserver, ln);
+					i_printfExit(MSG_EBRC_NoInserver, ln);
 				if (!act->outurl)
-					i_printfExit(MSG_ERBC_NoOutserver, ln);
+					i_printfExit(MSG_EBRC_NoOutserver, ln);
 				if (!act->login)
-					i_printfExit(MSG_ERBC_NoLogin, ln);
+					i_printfExit(MSG_EBRC_NoLogin, ln);
 				if (!act->password)
-					i_printfExit(MSG_ERBC_NPasswd, ln);
+					i_printfExit(MSG_EBRC_NPasswd, ln);
 				if (!act->from)
-					i_printfExit(MSG_ERBC_NoFrom, ln);
+					i_printfExit(MSG_EBRC_NoFrom, ln);
 				if (!act->reply)
-					i_printfExit(MSG_ERBC_NoReply, ln);
+					i_printfExit(MSG_EBRC_NoReply, ln);
+				if (act->secure)
+					act->inssl = act->outssl = 1;
 				if (!act->inport)
-					act->inport = 110;
+					if (act->secure)
+						act->inport =
+						    (act->imap ? 993 : 995);
+					else
+						act->inport =
+						    (act->imap ? 143 : 110);
 				if (!act->outport)
-					act->outport = 25;
+					act->outport = (act->secure ? 465 : 25);
 				continue;
 			}
 
@@ -719,34 +686,34 @@ nokeyword:
 				continue;
 			}
 
-			if (mimeblock == 1) {
+			if (mimeblock) {
 				++maxMime;
-				mimeblock = 0;
+				mimeblock = false;
 				if (!mt->type)
-					i_printfExit(MSG_ERBC_NoType, ln);
+					i_printfExit(MSG_EBRC_NoType, ln);
 				if (!mt->desc)
-					i_printfExit(MSG_ERBC_NDesc, ln);
+					i_printfExit(MSG_EBRC_NDesc, ln);
 				if (!mt->suffix && !mt->prot)
-					i_printfExit(MSG_ERBC_NoSuffix, ln);
+					i_printfExit(MSG_EBRC_NoSuffix, ln);
 				if (!mt->program)
-					i_printfExit(MSG_ERBC_NoProgram, ln);
+					i_printfExit(MSG_EBRC_NoProgram, ln);
 				continue;
 			}
 
-			if (tabblock == 1) {
+			if (tabblock) {
 				++maxTables;
-				tabblock = 0;
+				tabblock = false;
 				if (!td->name)
-					i_printfExit(MSG_ERBC_NoTblName, ln);
+					i_printfExit(MSG_EBRC_NoTblName, ln);
 				if (!td->shortname)
-					i_printfExit(MSG_ERBC_NoShortName, ln);
+					i_printfExit(MSG_EBRC_NoShortName, ln);
 				if (!td->ncols)
-					i_printfExit(MSG_ERBC_NColumns, ln);
+					i_printfExit(MSG_EBRC_NColumns, ln);
 				continue;
 			}
 
 			if (--nest < 0)
-				i_printfExit(MSG_ERBC_UnexpBrace, ln);
+				i_printfExit(MSG_EBRC_UnexpBrace, ln);
 			if (nest)
 				goto putback;
 /* This ends the function */
@@ -759,13 +726,13 @@ nokeyword:
 /* Does else make sense here? */
 			c = toupper(stack[nest]);
 			if (c != 'I')
-				i_printfExit(MSG_ERBC_UnexElse, ln);
+				i_printfExit(MSG_EBRC_UnexElse, ln);
 			goto putback;
 		}
 
 		if (*s != '\x81') {
 			if (!nest)
-				i_printfExit(MSG_ERBC_GarblText, ln);
+				i_printfExit(MSG_EBRC_GarblText, ln);
 			goto putback;
 		}
 
@@ -778,33 +745,33 @@ nokeyword:
 			if (mailblock > 1)
 				curblock = "a filter block";
 			if (mimeblock)
-				curblock = "a mime descriptor";
-			i_printfExit(MSG_ERBC_FnNotStart, ln, curblock);
+				curblock = "a plugin descriptor";
+			i_printfExit(MSG_EBRC_FnNotStart, ln, curblock);
 		}
 
 		if (!strchr("fmertsb", c) && !nest)
-			i_printfExit(MSG_ERBC_StatNotInFn, ln);
+			i_printfExit(MSG_EBRC_StatNotInFn, ln);
 
 		if (c == 'm') {
 			mailblock = 1;
 			if (maxAccount == MAXACCOUNT)
-				i_printfExit(MSG_ERBC_ManyAcc, MAXACCOUNT);
+				i_printfExit(MSG_EBRC_ManyAcc, MAXACCOUNT);
 			act = accounts + maxAccount;
 			continue;
 		}
 
 		if (c == 'e') {
-			mimeblock = 1;
+			mimeblock = true;
 			if (maxMime == MAXMIME)
-				i_printfExit(MSG_ERBC_ManyTypes, MAXMIME);
+				i_printfExit(MSG_EBRC_ManyTypes, MAXMIME);
 			mt = mimetypes + maxMime;
 			continue;
 		}
 
 		if (c == 'b') {
-			tabblock = 1;
+			tabblock = true;
 			if (maxTables == MAXDBT)
-				i_printfExit(MSG_ERBC_ManyTables, MAXDBT);
+				i_printfExit(MSG_EBRC_ManyTables, MAXDBT);
 			td = dbtables + maxTables;
 			continue;
 		}
@@ -827,7 +794,7 @@ nokeyword:
 		if (c == 'f') {
 			stack[++nest] = c;
 			if (sn == MAXEBSCRIPT)
-				i_printfExit(MSG_ERBC_ManyFn, sn);
+				i_printfExit(MSG_EBRC_ManyFn, sn);
 			ebScriptName[sn] = s + 2;
 			t[-1] = 0;
 			ebScript[sn] = t;
@@ -835,7 +802,7 @@ nokeyword:
 		}
 
 		if (++nest >= sizeof(stack))
-			i_printfExit(MSG_ERBC_TooDeeply, ln);
+			i_printfExit(MSG_EBRC_TooDeeply, ln);
 		stack[nest] = c;
 
 putback:
@@ -843,13 +810,10 @@ putback:
 	}			/* loop over lines */
 
 	if (nest)
-		i_printfExit(MSG_ERBC_FnNotClosed, ebScriptName[sn]);
+		i_printfExit(MSG_EBRC_FnNotClosed, ebScriptName[sn]);
 
 	if (mailblock | mimeblock)
-		i_printfExit(MSG_ERBC_MNotClosed);
-
-	if (cfgmodify)
-		updateConfig();
+		i_printfExit(MSG_EBRC_MNotClosed);
 }				/* readConfigFile */
 
 /*********************************************************************
@@ -876,9 +840,10 @@ const char *mailRedirect(const char *to, const char *from,
 	for (f = filters; f->match; ++f) {
 		const char *m = f->match;
 		int mlen = strlen(m);
-		r = f->redirect;
 		int j, k;
 
+		r = f->redirect;
+
 		switch (f->type) {
 		case 2:
 			if (stringEqualCI(m, from))
@@ -905,7 +870,7 @@ const char *mailRedirect(const char *to, const char *from,
 			if (slen < 16 || mlen < 16)
 				break;	/* too short */
 			j = k = 0;
-			while (eb_true) {
+			while (true) {
 				char c = subj[j];
 				char d = m[k];
 				if (isupperByte(c))
@@ -937,17 +902,19 @@ const char *mailRedirect(const char *to, const char *from,
 Are we ok to parse and execute javascript?
 *********************************************************************/
 
-eb_bool javaOK(const char *url)
+bool javaOK(const char *url)
 {
 	int j, hl, dl;
 	const char *h, *d, *q, *path;
 	if (!allowJS)
-		return eb_false;
+		return false;
 	if (!url)
-		return eb_true;
+		return true;
+	if (isDataURI(url))
+		return true;
 	h = getHostURL(url);
 	if (!h)
-		return eb_true;
+		return true;
 	hl = strlen(h);
 	path = getDataURL(url);
 	for (j = 0; j < javaDisCount; ++j) {
@@ -968,20 +935,20 @@ eb_bool javaOK(const char *url)
 				continue;
 			if (strncmp(q, path, strlen(q)))
 				continue;
-			return eb_false;
+			return false;
 		}		/* domain/path was specified */
 		if (hl == dl)
-			return eb_false;
+			return false;
 		if (h[hl - dl - 1] == '.')
-			return eb_false;
+			return false;
 	}
-	return eb_true;
+	return true;
 }				/* javaOK */
 
 /* Catch interrupt and react appropriately. */
 static void catchSig(int n)
 {
-	intFlag = eb_true;
+	intFlag = true;
 	if (inInput)
 		i_puts(MSG_EnterInterrupt);
 /* If we were reading from a file, or socket, this signal should
@@ -993,15 +960,53 @@ static void catchSig(int n)
 
 void ebClose(int n)
 {
+	bg_jobs(true);
 	dbClose();
+	js_shutdown();
+/* We should call curl_global_cleanup() here, for clarity and completeness,
+ * but it can cause a seg fault when combined with older versions of open ssl,
+ * and the process is going to exit anyways, so don't worry about it. */
 	exit(n);
 }				/* ebClose */
 
-void eeCheck(void)
+bool isSQL(const char *s)
 {
-	if (errorExit)
-		ebClose(1);
-}
+	char c;
+	const char *c1 = 0, *c2 = 0;
+	c = *s;
+
+	if (!sqlPresent)
+		goto no;
+
+	if (isURL(s))
+		goto no;
+
+	if (!isalphaByte(c))
+		goto no;
+
+	for (++s; c = *s; ++s) {
+		if (c == '_')
+			continue;
+		if (isalnumByte(c))
+			continue;
+		if (c == ':') {
+			if (c1)
+				goto no;
+			c1 = s;
+			continue;
+		}
+		if (c == ']') {
+			c2 = s;
+			goto yes;
+		}
+	}
+
+no:
+	return false;
+
+yes:
+	return true;
+}				/* isSQL */
 
 void setDataSource(char *v)
 {
@@ -1023,6 +1028,26 @@ void setDataSource(char *v)
 	dbpw = v;
 }				/* setDataSource */
 
+static void eb_curl_global_init(void)
+{
+	const unsigned int major = 7;
+	const unsigned int minor = 29;
+	const unsigned int patch = 0;
+	const unsigned int least_acceptable_version =
+	    (major << 16) | (minor << 8) | patch;
+	curl_version_info_data *version_data = NULL;
+	CURLcode curl_init_status = curl_global_init(CURL_GLOBAL_ALL);
+	if (curl_init_status != 0)
+		i_printfExit(MSG_LibcurlNoInit);
+	version_data = curl_version_info(CURLVERSION_NOW);
+	if (version_data->version_num < least_acceptable_version)
+		i_printfExit(MSG_CurlVersion, major, minor, patch);
+}				/* eb_curl_global_init */
+
+/*\ MSVC Debug: May need to provide path to 3rdParty DLLs, like
+ *  set PATH=F:\Projects\software\bin;%PATH% ...
+\*/
+
 /* I'm not going to expand wild card arguments here.
  * I don't need to on Unix, and on Windows there is a
  * setargv.obj, or something like that, that performs the expansion.
@@ -1033,17 +1058,21 @@ void setDataSource(char *v)
 int main(int argc, char **argv)
 {
 	int cx, account;
-	eb_bool rc, doConfig = eb_true;
-	eb_bool dofetch = eb_false, domail = eb_false;
+	bool rc, doConfig = true;
+	bool dofetch = false, domail = false;
 
+#ifndef _MSC_VER		// port setlinebuf(stdout);, if required...
 /* In case this is being piped over to a synthesizer, or whatever. */
 	if (fileTypeByHandle(fileno(stdout)) != 'f')
 		setlinebuf(stdout);
+#endif // !_MSC_VER
 
 	selectLanguage();
 
+	eb_curl_global_init();
 	ttySaveSettings();
 	initializeReadline();
+	progname = argv[0];
 
 /* Let's everybody use my malloc and free routines */
 	pcre_malloc = allocMem;
@@ -1051,31 +1080,58 @@ int main(int argc, char **argv)
 
 /* Establish the home directory, and standard edbrowse files thereunder. */
 	home = getenv("HOME");
+#ifdef _MSC_VER
+	if (!home) {
+		home = getenv("APPDATA");
+		if (home) {
+			char *ebdata = (char *)allocMem(ABSPATH);
+			sprintf(ebdata, "%s\\edbrowse", home);
+			if (fileTypeByName(ebdata, false) != 'd') {
+				FILE *fp;
+				char *cfgfil;
+				if (mkdir(ebdata, 0700)) {
+					i_printfExit(MSG_NotHome);	// TODO: more appropriate exit message...
+				}
+				cfgfil = (char *)allocMem(ABSPATH);
+				sprintf(cfgfil, "%s\\.ebrc", ebdata);
+				fp = fopen(cfgfil, "w");
+				if (fp) {
+					fwrite(ebrc_string, 1,
+					       strlen(ebrc_string), fp);
+					fclose(fp);
+				}
+				i_printfExit(MSG_Personalize, cfgfil);
+
+			}
+			home = ebdata;
+		}
+	}
+#endif // !_MSC_VER
+
 /* Empty is the same as missing. */
 	if (home && !*home)
 		home = 0;
 /* I require this, though I'm not sure what this means for non-Unix OS's */
 	if (!home)
 		i_printfExit(MSG_NotHome);
-	if (fileTypeByName(home, eb_false) != 'd')
+	if (fileTypeByName(home, false) != 'd')
 		i_printfExit(MSG_NotDir, home);
 
-/* See sample.ebrc in this directory for a sample config file. */
 	configFile = allocMem(strlen(home) + 7);
 	sprintf(configFile, "%s/.ebrc", home);
+/* if not present then create it, as was done above */
+	if (fileTypeByName(configFile, false) == 0) {
+		int fh = creat(configFile, 0600);
+		if (fh >= 0) {
+			write(fh, ebrc_string, strlen(ebrc_string));
+			close(fh);
+			i_printfExit(MSG_Personalize, configFile);
+		}
+	}
 
 	recycleBin = allocMem(strlen(home) + 8);
 	sprintf(recycleBin, "%s/.Trash", home);
-
-	edbrowseTempFile = allocMem(strlen(recycleBin) + 8 + 6);
-/* The extra 6 is for the suffix */
-	sprintf(edbrowseTempFile, "%s/eb_tmp", recycleBin);
-	edbrowseTempPDF = allocMem(strlen(recycleBin) + 8);
-	sprintf(edbrowseTempPDF, "%s/eb_pdf", recycleBin);
-	edbrowseTempHTML = allocMem(strlen(recycleBin) + 13);
-	sprintf(edbrowseTempHTML, "%s/eb_pdf.html", recycleBin);
-
-	if (fileTypeByName(recycleBin, eb_false) != 'd') {
+	if (fileTypeByName(recycleBin, false) != 'd') {
 		if (mkdir(recycleBin, 0700)) {
 /* Don't want to abort here; we might be on a readonly filesystem.
  * Don't have a Trash directory and can't creat one; yet we should move on. */
@@ -1087,7 +1143,7 @@ int main(int argc, char **argv)
 	if (recycleBin) {
 		mailStash = allocMem(strlen(recycleBin) + 12);
 		sprintf(mailStash, "%s/rawmail", recycleBin);
-		if (fileTypeByName(mailStash, eb_false) != 'd') {
+		if (fileTypeByName(mailStash, false) != 'd') {
 			if (mkdir(mailStash, 0700)) {
 				free(mailStash);
 				mailStash = 0;
@@ -1113,7 +1169,7 @@ int main(int argc, char **argv)
 			*argv = configFile;
 		else
 			++argv, --argc;
-		doConfig = eb_false;
+		doConfig = false;
 	} else {
 		readConfigFile();
 		if (maxAccount && !localAccount)
@@ -1141,24 +1197,24 @@ int main(int argc, char **argv)
 		}
 
 		if (stringEqual(s, "e")) {
-			errorExit = eb_true;
+			errorExit = true;
 			continue;
 		}
 
 		if (*s == 'p')
-			++s, passMail = eb_true;
+			++s, passMail = true;
 
 		if (*s == 'm' || *s == 'f') {
 			if (!maxAccount)
 				i_printfExit(MSG_NoMailAcc);
 			if (*s == 'f') {
 				account = 0;
-				dofetch = eb_true;
+				dofetch = true;
 				++s;
 				if (*s == 'm')
-					domail = eb_true, ++s;
+					domail = true, ++s;
 			} else {
-				domail = eb_true;
+				domail = true;
 				++s;
 			}
 			if (isdigitByte(*s)) {
@@ -1167,8 +1223,8 @@ int main(int argc, char **argv)
 					i_printfExit(MSG_BadAccNb, maxAccount);
 			}
 			if (!*s) {
-				ismc = eb_true;	/* running as a mail client */
-				allowJS = eb_false;	/* no javascript in mail client */
+				ismc = true;	/* running as a mail client */
+				allowJS = false;	/* no javascript in mail client */
 				++argv, --argc;
 				if (!argc || !dofetch)
 					break;
@@ -1187,8 +1243,6 @@ int main(int argc, char **argv)
 
 	srand(time(0));
 
-	loadBlacklist();
-
 	if (ismc) {
 		char **reclist, **atlist;
 		char *s, *body;
@@ -1199,6 +1253,9 @@ int main(int argc, char **argv)
 			if (dofetch) {
 				int nfetch = 0;
 				if (account) {
+					isimap = accounts[account - 1].imap;
+					if (isimap)
+						domail = false;
 					nfetch = fetchMail(account);
 				} else {
 					nfetch = fetchAllMail();
@@ -1243,18 +1300,20 @@ int main(int argc, char **argv)
 		memmove(reclist, reclist + 1, sizeof(char *) * nrec);
 		atlist[-1] = 0;
 		if (sendMail(account, (const char **)reclist, body, 1,
-			     (const char **)atlist, 0, nalt, eb_true))
+			     (const char **)atlist, 0, nalt, true))
 			exit(0);
 		showError();
 		exit(1);
 	}
 
 	cookiesFromJar();
-	my_curl_init();
+	http_curl_init();
 
 	signal(SIGINT, catchSig);
+#ifndef _MSC_VER		// port siginterrupt(SIGINT, 1); signal(SIGPIPE, SIG_IGN);, if required
 	siginterrupt(SIGINT, 1);
 	signal(SIGPIPE, SIG_IGN);
+#endif // !_MSC_VER
 
 	cx = 0;
 	while (argc) {
@@ -1262,15 +1321,15 @@ int main(int argc, char **argv)
 		++cx;
 		if (cx == MAXSESSION)
 			i_printfExit(MSG_ManyOpen, MAXSESSION);
-		cxSwitch(cx, eb_false);
+		cxSwitch(cx, false);
 		if (cx == 1)
 			runEbFunction("init");
 		changeFileName = 0;
 		cw->fileName = cloneString(file);
 		cw->firstURL = cloneString(file);
 		if (isSQL(file))
-			cw->sqlMode = eb_true;
-		rc = readFile(file, "");
+			cw->sqlMode = true;
+		rc = readFileArgv(file);
 		if (fileSize >= 0)
 			debugPrint(1, "%d", fileSize);
 		fileSize = -1;
@@ -1282,10 +1341,11 @@ int main(int argc, char **argv)
 			changeFileName = 0;
 		}
 
-		cw->undoable = cw->changeMode = eb_false;
+		cw->undoable = cw->changeMode = false;
 /* Browse the text if it's a url */
-		if (rc && !(cw->binMode | cw->dirMode) && cw->dol &&
-		    isBrowseableURL(cw->fileName)) {
+		if (rc && isURL(cw->fileName) &&
+		    (cw->mt && cw->mt->outtype
+		     || isBrowseableURL(cw->fileName))) {
 			if (runCommand("b"))
 				debugPrint(1, "%d", fileSize);
 			else
@@ -1295,22 +1355,24 @@ int main(int argc, char **argv)
 	}			/* loop over files */
 	if (!cx) {		/* no files */
 		++cx;
-		cxSwitch(cx, eb_false);
+		cxSwitch(cx, false);
 		runEbFunction("init");
 		i_puts(MSG_Ready);
 	}
 	if (cx > 1)
-		cxSwitch(1, eb_false);
+		cxSwitch(1, false);
 
-	while (eb_true) {
-		uchar saveline[MAXTTYLINE];
+	while (true) {
 		pst p = inputLine();
-		copyPstring(saveline, p);
-		if (perl2c((char *)p))
+		pst save_p = clonePstring(p);
+		if (perl2c((char *)p)) {
 			i_puts(MSG_EnterNull);
-		else
-			edbrowseCommand((char *)p, eb_false);
-		copyPstring(linePending, saveline);
+			nzFree(save_p);
+		} else {
+			edbrowseCommand((char *)p, false);
+			nzFree(linePending);
+			linePending = save_p;
+		}
 	}			/* infinite loop */
 }				/* main */
 
@@ -1320,7 +1382,7 @@ static const char *balance(const char *ip, int direction)
 	int nest = 0;
 	uchar code;
 
-	while (eb_true) {
+	while (true) {
 		if (direction > 0) {
 			ip = strchr(ip, '\n') + 1;
 		} else {
@@ -1345,7 +1407,7 @@ static const char *balance(const char *ip, int direction)
 }				/* balance */
 
 /* Run an edbrowse function, as defined in the config file. */
-eb_bool runEbFunction(const char *line)
+bool runEbFunction(const char *line)
 {
 	char *linecopy = cloneString(line);
 	const char *args[10];
@@ -1356,13 +1418,13 @@ eb_bool runEbFunction(const char *line)
 	int j, l, nest;
 	const char *ip;		/* think instruction pointer */
 	const char *endl;	/* end of line to be processed */
-	eb_bool nofail, ok;
+	bool nofail, ok;
 	uchar code;
 	char stack[MAXNEST];
 	int loopcnt[MAXNEST];
 
 /* Separate function name and arguments */
-	spaceCrunch(linecopy, eb_true, eb_false);
+	spaceCrunch(linecopy, true, false);
 	if (linecopy[0] == 0) {
 		setError(MSG_NoFunction);
 		goto fail;
@@ -1390,7 +1452,7 @@ eb_bool runEbFunction(const char *line)
 	ip = ebScript[j] + 1;
 	nofail = (ebScriptName[j][0] == '+');
 	nest = 0;
-	ok = eb_true;
+	ok = true;
 
 /* collect arguments */
 	j = 0;
@@ -1435,12 +1497,12 @@ eb_bool runEbFunction(const char *line)
 				continue;
 			}
 			if (ucontrol == 'W' || ucontrol == 'U') {
-				eb_bool jump = ok;
+				bool jump = ok;
 				if (islowerByte(control))
-					jump ^= eb_true;
+					jump ^= true;
 				if (ucontrol == 'U')
-					jump ^= eb_true;
-				ok = eb_true;
+					jump ^= true;
+				ok = true;
 				if (jump)
 					ip = start;
 				else
@@ -1453,7 +1515,7 @@ eb_bool runEbFunction(const char *line)
 
 		if (code == 0x81) {
 			const char *skip = balance(ip, 1);
-			eb_bool jump;
+			bool jump;
 			char control = ip[1];
 			char ucontrol = toupper(control);
 			stack[++nest] = control;
@@ -1472,8 +1534,8 @@ ahead:
 /* if or while, test on ok */
 			jump = ok;
 			if (isupperByte(control))
-				jump ^= eb_true;
-			ok = eb_true;
+				jump ^= true;
+			ok = true;
 			if (jump)
 				goto ahead;
 			goto nextline;
@@ -1516,7 +1578,7 @@ ahead:
 
 /* Here we go! */
 		debugPrint(3, "< %s", new);
-		ok = edbrowseCommand(new, eb_true);
+		ok = edbrowseCommand(new, true);
 		free(new);
 
 nextline:
@@ -1527,61 +1589,13 @@ nextline:
 		goto fail;
 
 	nzFree(linecopy);
-	return eb_true;
+	return true;
 
 fail:
 	nzFree(linecopy);
-	return eb_false;
+	return false;
 }				/* runEbFunction */
 
-/* Send the contents of the current buffer to a running program */
-eb_bool
-bufferToProgram(const char *cmd, const char *suffix, eb_bool trailPercent)
-{
-	char *buf = 0;
-	int buflen, n;
-	int size1, size2;
-	char *u = edbrowseTempFile + strlen(edbrowseTempFile);
-
-	if (!trailPercent) {
-/* pipe the buffer into the program */
-		FILE *f = popen(cmd, "w");
-		if (!f) {
-			setError(MSG_NoSpawn, cmd, errno);
-			return eb_false;
-		}
-		if (!unfoldBuffer(context, eb_false, &buf, &buflen)) {
-			pclose(f);
-			return eb_false;	/* should never happen */
-		}
-		n = fwrite(buf, buflen, 1, f);
-		pclose(f);
-	} else {
-		sprintf(u, ".%s", suffix);
-		size1 = currentBufferSize();
-		size2 = fileSizeByName(edbrowseTempFile);
-		if (size1 == size2) {
-/* assume it's the same data */
-			*u = 0;
-		} else {
-			if (!unfoldBuffer(context, eb_false, &buf, &buflen)) {
-				*u = 0;
-				return eb_false;	/* should never happen */
-			}
-			if (!memoryOutToFile(edbrowseTempFile, buf, buflen,
-					     MSG_TempNoCreate2, MSG_NoWrite2)) {
-				*u = 0;
-				return eb_false;
-			}
-			*u = 0;
-		}
-		system(cmd);
-	}
-
-	nzFree(buf);
-	return eb_true;
-}				/* bufferToProgram */
-
 struct DBTABLE *findTableDescriptor(const char *sn)
 {
 	int i;
@@ -1604,101 +1618,3 @@ struct DBTABLE *newTableDescriptor(const char *name)
 	td->ncols = 0;		/* it's already 0 */
 	return td;
 }				/* newTableDescriptor */
-
-struct MIMETYPE *findMimeBySuffix(const char *suffix)
-{
-	int i;
-	int len = strlen(suffix);
-	struct MIMETYPE *m = mimetypes;
-
-	for (i = 0; i < maxMime; ++i, ++m) {
-		const char *s = m->suffix, *t;
-		if (!s)
-			continue;
-		while (*s) {
-			t = strchr(s, ',');
-			if (!t)
-				t = s + strlen(s);
-			if (t - s == len && memEqualCI(s, suffix, len))
-				return m;
-			if (*t)
-				++t;
-			s = t;
-		}
-	}
-
-	return 0;
-}				/* findMimeBySuffix */
-
-struct MIMETYPE *findMimeByProtocol(const char *prot)
-{
-	int i;
-	int len = strlen(prot);
-	struct MIMETYPE *m = mimetypes;
-
-	for (i = 0; i < maxMime; ++i, ++m) {
-		const char *s = m->prot, *t;
-		if (!s)
-			continue;
-		while (*s) {
-			t = strchr(s, ',');
-			if (!t)
-				t = s + strlen(s);
-			if (t - s == len && memEqualCI(s, prot, len))
-				return m;
-			if (*t)
-				++t;
-			s = t;
-		}
-	}
-
-	return 0;
-}				/* findMimeByProtocol */
-
-/* The result is allocated */
-char *pluginCommand(const struct MIMETYPE *m, const char *file,
-		    const char *suffix)
-{
-	int len, suflen;
-	const char *s;
-	char *cmd, *t;
-	eb_bool trailPercent = eb_false;
-
-/* leave room for space quote quote null */
-	len = strlen(m->program) + 4;
-	if (file) {
-		len += strlen(file);
-	} else if (m->program[strlen(m->program) - 1] == '%') {
-		trailPercent = eb_true;
-		len += strlen(edbrowseTempFile) + 6;
-	}
-
-	suflen = 0;
-	if (suffix) {
-		suflen = strlen(suffix);
-		for (s = m->program; *s; ++s)
-			if (*s == '*')
-				len += suflen - 1;
-	}
-
-	cmd = allocMem(len);
-	t = cmd;
-	for (s = m->program; *s; ++s) {
-		if (suffix && *s == '*') {
-			strcpy(t, suffix);
-			t += suflen;
-		} else {
-			*t++ = *s;
-		}
-	}
-	*t = 0;
-
-	if (file) {
-		sprintf(t, " \"%s\"", file);
-	} else if (trailPercent) {
-		sprintf(t - 1, " \"%s.%s\"", edbrowseTempFile, suffix);
-	}
-
-	debugPrint(3, "%s", cmd);
-	return cmd;
-}				/* pluginCommand */
