@@ -1,16 +1,43 @@
 /* messages.h
- * Copyright (c) Karl Dahlke, 2014
  * This file is part of the edbrowse project, released under GPL.
  */
 
 /*********************************************************************
 Symbolic constants for message numbers.
-These become the indexes for the messages in message.c.
-The array of messages is selected by the language.
+These become the indexes for the messages in lang/msg-en,
+or in lang/msg-xx for language xx.
+The externs below correspond to the languages supported.
+The array of messages is selected by $LANG.
+See selectLanguage() in messages.c.
+EdbrowseMessageCount is the number of edbrowse messages, including some
+blank spaces for messages no longer used.
+Each file lang/msg-* must have exactly this many lines.
+*********************************************************************/
 
-This file and messages.c line up.
-In other words, the constant at line 100 of messages.h corresponds
-to the English string at line 100 of messages.c.
+#define EdbrowseMessageCount 659
+
+// English
+extern const char *msg_en[], *ebrc_en;
+
+// French, Erwin Bliesenick, erwinb@no-log.org
+extern const char *msg_fr[], *ebrc_fr;
+
+// Brazilian Portuguese, Cleverson Casarin, clcaul@gmail.com
+extern const char *msg_pt_br[], *ebrc_pt_br;
+
+// German, Sebastian Humenda, shumenda@gmx.de
+extern const char *msg_de[], *ebrc_de;
+
+// Polish, Wojciech Gac, wojciech.s.gac@gmail.com
+extern const char *msg_pl[], *ebrc_pl;
+
+// Russian, Wojciech Gac, wojciech.s.gac@gmail.com
+extern const char *msg_ru[], *ebrc_ru;
+
+/*********************************************************************
+This file and lang/msg-en line up.
+The constant at line 101 of messages.h corresponds
+to the English string at line 1 of lang/msg-en.
 to keep this alignment, even if you run this file through indent,
 this comment block is just the right size.
 *
@@ -20,12 +47,60 @@ this comment block is just the right size.
 *
 *
 *
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
+*
 *********************************************************************/
 
 enum {
 	MSG_EndFile,
 	MSG_NoFile,
-	MSG_None,
+	MSG_Redirect,
 	MSG_SubGlobal,
 	MSG_SubLocal,
 	MSG_CaseIns,
@@ -43,13 +118,13 @@ enum {
 	MSG_BinaryDetect,
 	MSG_PassiveMode,
 	MSG_ActiveMode,
-	MSG_NotUsed2,
+	MSG_JSCloseSessions,
 	MSG_CertifyOff,
 	MSG_CertifyOn,
 	MSG_HiddenOff,
 	MSG_HiddenOn,
-	MSG_NotUsed3,
-	MSG_NotUsed4,
+	MSG_JSEngineFork,
+	MSG_JSEngineExec,
 	MSG_MarkOff,
 	MSG_MarkList,
 	MSG_MarkOn,
@@ -68,10 +143,10 @@ enum {
 	MSG_NoKeywords,
 	MSG_SessionX,
 	MSG_MailHowto,
-	MSG_LineUpdated,
-	MSG_NoSubject,
-	MSG_NoConfig,
-	MSG_NoSubjFilter,
+	MSG_LineUpdate1,
+	MSG_ImapMessageHelp,
+	MSG_ImapSearchHelp,
+	MSG_NoMessages,
 	MSG_EnterInterrupt,
 	MSG_Ready,
 	MSG_EnterNull,
@@ -84,16 +159,16 @@ enum {
 	MSG_UserNameLong,
 	MSG_Password,
 	MSG_PasswordLong,
-	MSG_NotUsed5,
+	MSG_JSEnginePipe,
 	MSG_NoDirNoList,
 	MSG_ErrorMessageLong,
 	MSG_LineX,
-	MSG_BrowseError,
-	MSG_NoSideBuffer,
+	MSG_NoDir,
+	MSG_BufferUpdated,
 	MSG_NoOptions,
 	MSG_FormReset,
 	MSG_FormSubmit,
-	MSG_SideBufferX,
+	MSG_EBRC_Outtype,
 	MSG_Many,
 	MSG_Recommended,
 	MSG_Close,
@@ -104,9 +179,9 @@ enum {
 	MSG_Quit,
 	MSG_Next,
 	MSG_Delete,
-	MSG_IPDelete,
-	MSG_POption,
-	MSG_NoBlackList,
+	MSG_PluginsOff,
+	MSG_PluginsOn,
+	MSG_JSEngineRun,
 	MSG_EndMessage,
 	MSG_NYI,
 	MSG_FileName,
@@ -120,15 +195,15 @@ enum {
 	MSG_AttNoWrite,
 	MSG_NoMail,
 	MSG_MessagesX,
-	MSG_Spam,
-	MSG_From,
+	MSG_JSEngineSync,
+	MSG_LSBadChar,
 	MSG_MailHelp,
 	MSG_NoCreate,
 	MSG_NoWrite,
 	MSG_MailSaved,
 	MSG_Appended,
 	MSG_PageDone,
-	MSG_InnerText,
+	MSG_TempDir,
 	MSG_NoErrors,
 	MSG_ProxyAuth,
 	MSG_SessionInactive,
@@ -182,7 +257,7 @@ enum {
 	MSG_NotModifiedG,
 	MSG_Interrupted,
 	MSG_NoMatch,
-	MSG_NotUsed6,
+	MSG_JSEngineVars,
 	MSG_NoAgent,
 	MSG_CDGetError,
 	MSG_CDSetError,
@@ -205,7 +280,7 @@ enum {
 	MSG_BalanceAmbig,
 	MSG_BalanceNothing,
 	MSG_Unbalanced,
-	MSG_SessionBrowse,
+	MSG_Personalize,
 	MSG_SessionDir,
 	MSG_NoFileName,
 	MSG_EndBuffer,
@@ -259,7 +334,7 @@ enum {
 	MSG_NoLable2,
 	MSG_BrowseI,
 	MSG_InsertFunction,
-	MSG_NotUsed7,
+	MSG_JSEngineRW,
 	MSG_CNYI,
 	MSG_XOutOfRange,
 	MSG_OptMatchNone,
@@ -288,7 +363,7 @@ enum {
 	MSG_InputRange,
 	MSG_InputRange2,
 	MSG_ManyEmptyStrings,
-	MSG_NotUsed8,
+	MSG_DownAbort,
 	MSG_RexpMissing2,
 	MSG_SubNumbersMany,
 	MSG_SubSuffixBad,
@@ -309,36 +384,36 @@ enum {
 	MSG_ManyButtons,
 	MSG_WebRead,
 	MSG_BadURL,
-	MSG_NotUsed9,
+	MSG_DownSuccess,
 	MSG_IdentifyHost,
 	MSG_WebProtBad,
 	MSG_BufferPreload,
-	MSG_NotUsed10,
+	MSG_DownProgress,
 	MSG_WebConnect,
-	MSG_NotUsed11,
+	MSG_Down,
 	MSG_NoCertify,
-	MSG_NotUsed12,
-	MSG_NotUsed13,
+	MSG_Inaccess,
+	MSG_ImapReadHelp,
 	MSG_Authorize2,
 	MSG_LoginAbort,
-	MSG_NotUsed15,
-	MSG_TempNoWrite,
-	MSG_NotUsed14,
-	MSG_NotUsed16,
-	MSG_NotUsed17,
-	MSG_NotUsed18,
+	MSG_SearchQuote,
+	MSG_EndFolder,
+	MSG_DownForeground,
+	MSG_DownBackground,
+	MSG_Complete,
+	MSG_Failed,
 	MSG_FTPConnect,
-	MSG_FTPConnectTime,
 	MSG_FTPTransfer,
-	MSG_FTPTransferTime,
-	MSG_NotUsed49,
-	MSG_NotUsed19,
-	MSG_FTPBadURL,
-	MSG_NotUsed20,
-	MSG_NotUsed1,
-	MSG_NotUsed21,
+	MSG_NoFolders,
+	MSG_SelectFolder,
+	MSG_InProgress,
+	MSG_Stop,
+	MSG_FetchN,
+	MSG_NoAttachments,
+	MSG_EBRC_KeyInFunc,
+	MSG_ShowLast,
 	MSG_FTPSession,
-	MSG_NotUsed22,
+	MSG_Push,
 	MSG_DBNotCompiled,
 	MSG_ABNoAlias,
 	MSG_ABNoColon,
@@ -348,17 +423,17 @@ enum {
 	MSG_ABMailSpaces,
 	MSG_AbMailUnprintable,
 	MSG_ABUnterminated,
-	MSG_NotUsed23,
-	MSG_NotUsed24,
-	MSG_NotUsed25,
-	MSG_NotUsed26,
-	MSG_NotUsed27,
+	MSG_NoFolderMatch,
+	MSG_ManyFolderMatch,
+	MSG_Search,
+	MSG_Limit,
+	MSG_MoveTo,
 	MSG_SectionIgnored,
 	MSG_FileXEmpty,
 	MSG_SubjectStart,
-	MSG_NotUsed51,
+	MSG_notused334,
 	MSG_SubjectLong,
-	MSG_NotUsed28,
+	MSG_notused336,
 	MSG_SigRegular,
 	MSG_SigAccess,
 	MSG_MailBinary,
@@ -371,11 +446,11 @@ enum {
 	MSG_AttAccess,
 	MSG_AttRegular,
 	MSG_AttEmpty2,
-	MSG_NotUsed29,
-	MSG_NotUsed30,
-	MSG_MailReject,
-	MSG_NotUsed31,
-	MSG_NotUsed32,
+	MSG_LineDelete1,
+	MSG_LineDelete2,
+	MSG_LineAdd1,
+	MSG_LineAdd2,
+	MSG_notused353,
 	MSG_MailAccountsNone,
 	MSG_MailAccountBad,
 	MSG_MailBrowse,
@@ -387,9 +462,9 @@ enum {
 	MSG_MailFirstCC,
 	MSG_AttLineX,
 	MSG_MailAccountBadLineX,
-	MSG_NotUsed52,
+	MSG_notused365,
 	MSG_MailFirstLine,
-	MSG_NotUsed33,
+	MSG_notused367,
 	MSG_RecipNone3,
 	MSG_ProtExpected,
 	MSG_BadProt,
@@ -397,8 +472,8 @@ enum {
 	MSG_DomainLong,
 	MSG_UserNameLong2,
 	MSG_PasswordLong2,
-	MSG_NotUsed34,
-	MSG_NotUsed35,
+	MSG_LineUpdate2,
+	MSG_LineUpdate3,
 	MSG_NoFunction,
 	MSG_BadFunctionName,
 	MSG_NoSuchFunction,
@@ -408,18 +483,18 @@ enum {
 	MSG_TempNoCreate2,
 	MSG_ManyTables,
 	MSG_RegularFile,
-	MSG_NotUsed36,
+	MSG_notused386,
 	MSG_NoRead2,
 	MSG_NoAccess,
 	MSG_NoEnvVar,
-	MSG_EarlyExpand,
-	MSG_NoAccessDir,
-	MSG_ShellPatternLong,
-	MSG_ExpandBackslash,
-	MSG_ShellSyntax,
-	MSG_ShellCompile,
+	MSG_NoTilde,
+	MSG_Submit,
+	MSG_notused392,
+	MSG_Secure,
+	MSG_Bymail,
+	MSG_Reset,
 	MSG_ShellExpand,
-	MSG_ShellNoMatch,
+	MSG_notused397,
 	MSG_ShellManyMatch,
 	MSG_ShellLineLong,
 	MSG_DBUnspecified,
@@ -455,41 +530,41 @@ enum {
 	MSG_DBCheck,
 	MSG_DBTimeout,
 	MSG_DBView,
-	MSG_CloseTag,
-	MSG_JSKeystroke,
-	MSG_JSFocus,
-	MSG_StrayOnchange,
-	MSG_NotInForm2,
-	MSG_FieldNoName,
-	MSG_GetPost,
-	MSG_Enctype,
-	MSG_FormProtBad,
-	MSG_InputType,
-	MSG_RadioMany,
-	MSG_TagNest,
-	MSG_TagInTag,
-	MSG_TagClose,
-	MSG_TextareaNest,
-	MSG_InAnchor,
-	MSG_HasTags,
-	MSG_OptionComma,
-	MSG_OptionEmpty,
-	MSG_ManyTitles,
-	MSG_NotInList,
-	MSG_NotInTable,
-	MSG_NotInRow,
-	MSG_NotInSelect,
-	MSG_ManyOptSelected,
-	MSG_BadTag,
-	MSG_TagNotClosed,
-	MSG_JSBlankWindow,
+	MSG_notused433,
+	MSG_notused434,
+	MSG_notused435,
+	MSG_notused436,
+	MSG_notused437,
+	MSG_notused438,
+	MSG_notused439,
+	MSG_notused440,
+	MSG_notused441,
+	MSG_notused442,
+	MSG_notused443,
+	MSG_notused444,
+	MSG_notused445,
+	MSG_notused446,
+	MSG_notused447,
+	MSG_notused448,
+	MSG_notused449,
+	MSG_notused450,
+	MSG_notused451,
+	MSG_notused452,
+	MSG_notused453,
+	MSG_notused454,
+	MSG_notused455,
+	MSG_notused456,
+	MSG_notused457,
+	MSG_notused458,
+	MSG_notused459,
+	MSG_notused460,
 	MSG_AttAfterChars,
 	MSG_AttBad64,
-	MSG_OnchangeText,
+	MSG_notused463,
 	MSG_Doubleclick,
 	MSG_OptionSync,
-	MSG_UnloadClick,
-	MSG_ScriptNotClosed,
+	MSG_notused466,
+	MSG_notused467,
 	MSG_GetLocalJS,
 	MSG_GetJS,
 	MSG_GetJS2,
@@ -498,75 +573,75 @@ enum {
 	MSG_NJNoReset,
 	MSG_NJNoSubmit,
 	MSG_LostTag,
-	MSG_JSTextarea,
+	MSG_notused476,
 	MSG_GarbledRefresh,
-	MSG_NotUsed37,
+	MSG_notused478,
 	MSG_RedirectNoURL,
 	MSG_HTTPError,
-	MSG_NotUsed50,
-	MSG_ConfigUpdate,
+	MSG_notused481,
+	MSG_notused482,
 	MSG_HelpOn,
 	MSG_FTPDownload,
 	MSG_NoCertFile,
-	MSG_ERBC_NoWrite,
-	MSG_ERBC_Nulls,
-	MSG_ERBC_NoFnName,
-	MSG_ERBC_FnDigit,
-	MSG_ERBC_FnTooLong,
-	MSG_ERBC_SyntaxErr,
-	MSG_ERBC_NoCondFile,
-	MSG_ERBC_NoMatchStr,
-	MSG_ERBC_MatchNowh,
-	MSG_ERBC_Filters,
-	MSG_ERBC_BadKeyword,
-	MSG_ERBC_MailAttrOut,
-	MSG_ERBC_MimeAttrOut,
-	MSG_ERBC_TableAttrOut,
-	MSG_ERBC_MailAttrIn,
-	MSG_ERBC_MimeAttrIn,
-	MSG_ERBC_TableAttrIn,
-	MSG_ERBC_NoAttr,
-	MSG_ERBC_ManyCols,
-	MSG_ERBC_KeyNotNb,
-	MSG_ERBC_KeyOutRange,
-	MSG_ERBC_AbNotFile,
-	MSG_ERBC_IPNotFile,
-	MSG_ERBC_NotDir,
-	MSG_ERBC_ManyAgents,
-	MSG_ERBC_JarNotFile,
-	MSG_ERBC_JarNoWrite,
-	MSG_ERBC_NoJS,
-	MSG_ERBC_DomainDot,
-	MSG_ERBC_TrashNotFile,
-	MSG_ERBC_SSLNoFile,
-	MSG_ERBC_SSLNoRead,
-	MSG_ERBC_KeywordNYI,
-	MSG_ERBC_SevDefaults,
-	MSG_ERBC_NoInserver,
-	MSG_ERBC_NoOutserver,
-	MSG_ERBC_NoLogin,
-	MSG_ERBC_NPasswd,
-	MSG_ERBC_NoFrom,
-	MSG_ERBC_NoReply,
-	MSG_ERBC_NoType,
-	MSG_ERBC_NDesc,
-	MSG_ERBC_NoSuffix,
-	MSG_ERBC_NoProgram,
-	MSG_ERBC_NoTblName,
-	MSG_ERBC_NoShortName,
-	MSG_ERBC_NColumns,
-	MSG_ERBC_UnexpBrace,
-	MSG_ERBC_UnexElse,
-	MSG_ERBC_GarblText,
-	MSG_ERBC_FnNotStart,
-	MSG_ERBC_StatNotInFn,
-	MSG_ERBC_ManyAcc,
-	MSG_ERBC_ManyTypes,
-	MSG_ERBC_ManyTables,
-	MSG_ERBC_ManyFn,
-	MSG_ERBC_TooDeeply,
-	MSG_ERBC_FnNotClosed,
-	MSG_ERBC_MNotClosed,
+	MSG_SCPDownload,
+	MSG_EBRC_Nulls,
+	MSG_EBRC_NoFnName,
+	MSG_EBRC_FnDigit,
+	MSG_EBRC_FnTooLong,
+	MSG_EBRC_SyntaxErr,
+	MSG_EBRC_NoCondFile,
+	MSG_EBRC_NoMatchStr,
+	MSG_EBRC_MatchNowh,
+	MSG_EBRC_Filters,
+	MSG_EBRC_BadKeyword,
+	MSG_EBRC_MailAttrOut,
+	MSG_EBRC_MimeAttrOut,
+	MSG_EBRC_TableAttrOut,
+	MSG_EBRC_MailAttrIn,
+	MSG_EBRC_MimeAttrIn,
+	MSG_EBRC_TableAttrIn,
+	MSG_EBRC_NoAttr,
+	MSG_EBRC_ManyCols,
+	MSG_EBRC_KeyNotNb,
+	MSG_EBRC_KeyOutRange,
+	MSG_EBRC_AbNotFile,
+	MSG_notused508,
+	MSG_EBRC_NotDir,
+	MSG_EBRC_ManyAgents,
+	MSG_EBRC_JarNotFile,
+	MSG_EBRC_JarNoWrite,
+	MSG_EBRC_NoJS,
+	MSG_EBRC_DomainDot,
+	MSG_notused515,
+	MSG_EBRC_SSLNoFile,
+	MSG_EBRC_SSLNoRead,
+	MSG_EBRC_KeywordNYI,
+	MSG_EBRC_SevDefaults,
+	MSG_EBRC_NoInserver,
+	MSG_EBRC_NoOutserver,
+	MSG_EBRC_NoLogin,
+	MSG_EBRC_NPasswd,
+	MSG_EBRC_NoFrom,
+	MSG_EBRC_NoReply,
+	MSG_EBRC_NoType,
+	MSG_EBRC_NDesc,
+	MSG_EBRC_NoSuffix,
+	MSG_EBRC_NoProgram,
+	MSG_EBRC_NoTblName,
+	MSG_EBRC_NoShortName,
+	MSG_EBRC_NColumns,
+	MSG_EBRC_UnexpBrace,
+	MSG_EBRC_UnexElse,
+	MSG_EBRC_GarblText,
+	MSG_EBRC_FnNotStart,
+	MSG_EBRC_StatNotInFn,
+	MSG_EBRC_ManyAcc,
+	MSG_EBRC_ManyTypes,
+	MSG_EBRC_ManyTables,
+	MSG_EBRC_ManyFn,
+	MSG_EBRC_TooDeeply,
+	MSG_EBRC_FnNotClosed,
+	MSG_EBRC_MNotClosed,
 	MSG_NotHome,
 	MSG_NotDir,
 	MSG_NoMailAcc,
@@ -577,7 +652,7 @@ enum {
 	MSG_ManyOpen,
 	MSG_InvalidSession,
 	MSG_InvalidLineNb,
-	MSG_ERBC_NoPROXY,
+	MSG_EBRC_NoPROXY,
 	MSG_SessionOutRange,
 	MSG_DoubleInit,
 	MSG_QuitNoActive,
@@ -586,30 +661,30 @@ enum {
 	MSG_NoClosingLine,
 	MSG_NoTagFound,
 	MSG_NoRebCookie,
-	MSG_ManyIP,
+	MSG_notused564,
 	MSG_FetchNotBackgnd,
 	MSG_NoMailDir,
 	MSG_NoDirChange,
-	MSG_NotUsed39,
+	MSG_notused568,
 	MSG_LogPass,
-	MSG_NotUsed40,
-	MSG_NotUsed41,
-	MSG_NotUsed42,
-	MSG_NotUsed43,
-	MSG_NotUsed44,
+	MSG_notused570,
+	MSG_notused571,
+	MSG_notused572,
+	MSG_notused573,
+	MSG_notused574,
 	MSG_BadTagCode,
-	MSG_ErrCallPreFormat,
+	MSG_notused576,
 	MSG_HtmlNotreentrant,
 	MSG_UnexSubmitForm,
 	MSG_NullListInform,
-	MSG_NotUsed45,
+	MSG_notused580,
 	MSG_JavaMemError,
 	MSG_JavaContextError,
 	MSG_JavaWindowError,
-	MSG_JavaClassError,
+	MSG_notused584,
 	MSG_JavaObjError,
-	MSG_PortTooLong,
-	MSG_ProtTooLong,
+	MSG_notused586,
+	MSG_notused587,
 	MSG_MemAllocError,
 	MSG_MemCallocError,
 	MSG_ReallocP,
@@ -626,9 +701,9 @@ enum {
 	MSG_BadDirSlash,
 	MSG_BadSlash,
 	MSG_DecodePost,
-	MSG_NoPDF,
-	MSG_NotUsed46,
-	MSG_NotUsed47,
+	MSG_notused604,
+	MSG_notused605,
+	MSG_notused606,
 	MSG_ReDir,
 	MSG_ReDB,
 	MSG_ReEmpty,
@@ -678,7 +753,8 @@ enum {
 	MSG_SSLConnectError,
 	MSG_InputTTY,
 	MSG_InputReadLine,
-	MSG_NotUsed48,
+	MSG_notused656,
 	MSG_JSSessionFail,
-	MSG_StrayOnclick,
+	MSG_SystemCmdFail,
+	MSG_notused659,
 };
