@@ -1,33 +1,38 @@
 /* stringfile.c: manage strings, files, and directories for edbrowse.
-* Copyright (c) Karl Dahlke, 2008
 * This file is part of the edbrowse project, released under GPL.
  */
 
 #include "eb.h"
 
 #include <sys/stat.h>
-#include <arpa/inet.h>
+#include <dirent.h>
 #ifdef DOSLIKE
 #include <dos.h>
 #else
-#include <dirent.h>
 #include <termios.h>
 #include <unistd.h>
+#include <glob.h>
+#include <pwd.h>
+#include <grp.h>
 #endif
-#include <pcre.h>
-#include <netdb.h>
+
+char emptyString[] = "";
+bool showHiddenFiles, isInteractive;
+int debugLevel = 1;
+char *downDir, *home;
 
 /*********************************************************************
 Allocate and manage memory.
 Allocate and copy strings.
 If we're out of memory, the program aborts.  No error legs.
+Soooooo much easier! With 32gb of RAM, we shouldn't run out.
 *********************************************************************/
 
 void *allocMem(size_t n)
 {
 	void *s;
 	if (!n)
-		return EMPTYSTRING;
+		return emptyString;
 	if (!(s = malloc(n)))
 		i_printfExit(MSG_MemAllocError, n);
 	return s;
@@ -37,7 +42,7 @@ void *allocZeroMem(size_t n)
 {
 	void *s;
 	if (!n)
-		return EMPTYSTRING;
+		return emptyString;
 	if (!(s = calloc(n, 1)))
 		i_printfExit(MSG_MemCallocError, n);
 	return s;
@@ -48,21 +53,46 @@ void *reallocMem(void *p, size_t n)
 	void *s;
 	if (!n)
 		i_printfExit(MSG_ReallocP);
+/* small check against allocated strings getting huge */
+	if (n < 0)
+		i_printfExit(MSG_MemAllocError, n);
 	if (!p)
 		i_printfExit(MSG_Realloc0, n);
-	if (p == EMPTYSTRING)
+	if (p == emptyString)
 		return allocMem(n);
 	if (!(s = realloc(p, n)))
 		i_printfExit(MSG_ErrorRealloc, n);
 	return s;
 }				/* reallocMem */
 
+/* When you know the allocated thing is a string. */
+char *allocString(size_t n)
+{
+	return (char *)allocMem(n);
+}				/* allocString */
+
+char *allocZeroString(size_t n)
+{
+	return (char *)allocZeroMem(n);
+}				/* allocZeroString */
+
+char *reallocString(void *p, size_t n)
+{
+	return (char *)reallocMem(p, n);
+}				/* reallocString */
+
 void nzFree(void *s)
 {
-	if (s && s != EMPTYSTRING)
+	if (s && s != emptyString)
 		free(s);
 }				/* nzFree */
 
+/* some compilers care whether it's void * or const void * */
+void cnzFree(const void *v)
+{
+	nzFree((void *)v);
+}				/* cnzFree */
+
 uchar fromHex(char d, char e)
 {
 	d |= 0x20, e |= 0x20;
@@ -78,7 +108,7 @@ char *appendString(char *s, const char *p)
 {
 	int slen = strlen(s);
 	int plen = strlen(p);
-	s = reallocMem(s, slen + plen + 1);
+	s = reallocString(s, slen + plen + 1);
 	strcpy(s + slen, p);
 	return s;
 }				/* appendstring */
@@ -87,7 +117,7 @@ char *prependString(char *s, const char *p)
 {
 	int slen = strlen(s);
 	int plen = strlen(p);
-	char *t = allocMem(slen + plen + 1);
+	char *t = allocString(slen + plen + 1);
 	strcpy(t, p);
 	strcpy(t + plen, s);
 	nzFree(s);
@@ -102,36 +132,43 @@ void skipWhite(const char **s)
 	*s = t;
 }				/* skipWhite */
 
+void trimWhite(char *s)
+{
+	int l;
+	if (!s)
+		return;
+	l = strlen(s);
+	while (l && isspaceByte(s[l - 1]))
+		--l;
+	s[l] = 0;
+}				/* trimWhite */
+
 void stripWhite(char *s)
 {
 	const char *t = s;
-	char *u;
 	skipWhite(&t);
 	if (t > s)
 		strmove(s, t);
-	u = s + strlen(s);
-	while (u > s && isspaceByte(u[-1]))
-		--u;
-	*u = 0;
+	trimWhite(s);
 }				/* stripWhite */
 
 /* compress white space */
-void spaceCrunch(char *s, eb_bool onespace, eb_bool unprint)
+void spaceCrunch(char *s, bool onespace, bool unprint)
 {
 	int i, j;
 	char c;
-	eb_bool space = eb_true;
+	bool space = true;
 	for (i = j = 0; c = s[i]; ++i) {
 		if (isspaceByte(c)) {
 			if (!onespace)
 				continue;
 			if (!space)
-				s[j++] = ' ', space = eb_true;
+				s[j++] = ' ', space = true;
 			continue;
 		}
 		if (unprint && !isprintByte(c))
 			continue;
-		s[j++] = c, space = eb_false;
+		s[j++] = c, space = false;
 	}
 	if (space && j)
 		--j;		/* drop trailing space */
@@ -141,7 +178,7 @@ void spaceCrunch(char *s, eb_bool onespace, eb_bool unprint)
 /* Like strcpy, but able to cope with overlapping strings. */
 char *strmove(char *dest, const char *src)
 {
-	return memmove(dest, src, strlen(src) + 1);
+	return (char *)memmove(dest, src, strlen(src) + 1);
 }				/* strmove */
 
 /* OO has a lot of unnecessary overhead, and a few inconveniences,
@@ -153,9 +190,10 @@ char *strmove(char *dest, const char *src)
 char *initString(int *l)
 {
 	*l = 0;
-	return EMPTYSTRING;
+	return emptyString;
 }
 
+/* String management routines realloc to one less than a power of 2 */
 void stringAndString(char **s, int *l, const char *t)
 {
 	char *p = *s;
@@ -171,7 +209,7 @@ void stringAndString(char **s, int *l, const char *t)
 		newlen |= (newlen >> 4);
 		newlen |= (newlen >> 8);
 		newlen |= (newlen >> 16);
-		p = reallocMem(p, newlen);
+		p = reallocString(p, newlen);
 		*s = p;
 	}
 	strcpy(p + oldlen, t);
@@ -192,7 +230,7 @@ void stringAndBytes(char **s, int *l, const char *t, int cnt)
 		newlen |= (newlen >> 4);
 		newlen |= (newlen >> 8);
 		newlen |= (newlen >> 16);
-		p = reallocMem(p, newlen);
+		p = reallocString(p, newlen);
 		*s = p;
 	}
 	memcpy(p + oldlen, t, cnt);
@@ -214,7 +252,7 @@ void stringAndChar(char **s, int *l, char c)
 		newlen |= (newlen >> 4);
 		newlen |= (newlen >> 8);
 		newlen |= (newlen >> 16);
-		p = reallocMem(p, newlen);
+		p = reallocString(p, newlen);
 		*s = p;
 	}
 	p[oldlen] = c;
@@ -249,33 +287,21 @@ char *cloneString(const char *s)
 	if (!s)
 		return 0;
 	if (!*s)
-		return EMPTYSTRING;
+		return emptyString;
 	len = strlen(s) + 1;
-	t = allocMem(len);
+	t = allocString(len);
 	strcpy(t, s);
 	return t;
 }				/* cloneString */
 
 char *cloneMemory(const char *s, int n)
 {
-	char *t = allocMem(n);
+	char *t = allocString(n);
 	if (n)
 		memcpy(t, s, n);
 	return t;
 }				/* cloneMemory */
 
-void clipString(char *s)
-{
-	int len;
-	if (!s)
-		return;
-	len = strlen(s);
-	while (--len >= 0)
-		if (!isspaceByte(s[len]))
-			break;
-	s[len + 1] = 0;
-}				/* clipString */
-
 void leftClipString(char *s)
 {
 	char *t;
@@ -300,7 +326,7 @@ void shiftRight(char *s, char first)
 char *Cify(const char *s, int n)
 {
 	char *u;
-	char *t = allocMem(n + 1);
+	char *t = allocString(n + 1);
 	if (n)
 		memcpy(t, s, n);
 	for (u = t; u < t + n; ++u)
@@ -316,8 +342,8 @@ char *pullString(const char *s, int l)
 {
 	char *t;
 	if (!l)
-		return EMPTYSTRING;
-	t = allocMem(l + 1);
+		return emptyString;
+	t = allocString(l + 1);
 	memcpy(t, s, l);
 	t[l] = 0;
 	return t;
@@ -328,6 +354,7 @@ char *pullString1(const char *s, const char *t)
 	return pullString(s, t - s);
 }
 
+/* return the number, if string is a number, else -1 */
 int stringIsNum(const char *s)
 {
 	int n;
@@ -339,24 +366,24 @@ int stringIsNum(const char *s)
 	return n;
 }				/* stringIsNum */
 
-eb_bool stringIsDate(const char *s)
+bool stringIsDate(const char *s)
 {
 	if (!isdigit(*s))
-		return eb_false;
+		return false;
 	++s;
 	if (isdigit(*s))
 		++s;
 	if (*s++ != '/')
-		return eb_false;
+		return false;
 	if (!isdigit(*s))
-		return eb_false;
+		return false;
 	++s;
 	if (isdigit(*s))
 		++s;
 	if (*s++ != '/')
-		return eb_false;
+		return false;
 	if (!isdigit(*s))
-		return eb_false;
+		return false;
 	++s;
 	if (isdigit(*s))
 		++s;
@@ -365,69 +392,26 @@ eb_bool stringIsDate(const char *s)
 	if (isdigit(*s))
 		++s;
 	if (*s)
-		return eb_false;
-	return eb_true;
+		return false;
+	return true;
 }				/* stringIsDate */
 
-eb_bool stringIsFloat(const char *s, double *dp)
+bool stringIsFloat(const char *s, double *dp)
 {
 	const char *t;
 	*dp = strtod(s, (char **)&t);
 	if (*t)
-		return eb_false;	/* extra stuff at the end */
-	return eb_true;
+		return false;	/* extra stuff at the end */
+	return true;
 }				/* stringIsFloat */
 
-eb_bool stringIsPDF(const char *s)
-{
-	int j = 0;
-	if (s)
-		j = strlen(s);
-	return j >= 5 && stringEqual(s + j - 4, ".pdf");
-}				/* stringIsPDF */
-
-eb_bool isSQL(const char *s)
-{
-	char c;
-	const char *c1 = 0, *c2 = 0;
-	c = *s;
-
-	if (!sqlPresent)
-		goto no;
-
-	if (isURL(s))
-		goto no;
-
-	if (!isalphaByte(c))
-		goto no;
-
-	for (++s; c = *s; ++s) {
-		if (c == '_')
-			continue;
-		if (isalnumByte(c))
-			continue;
-		if (c == ':') {
-			if (c1)
-				goto no;
-			c1 = s;
-			continue;
-		}
-		if (c == ']') {
-			c2 = s;
-			goto yes;
-		}
-	}
-
-no:
-	return eb_false;
-
-yes:
-	return eb_true;
-}				/* isSQL */
-
-eb_bool memEqualCI(const char *s, const char *t, int len)
+bool memEqualCI(const char *s, const char *t, int len)
 {
 	char c, d;
+	if (s == t)
+		return true;
+	if (!s || !t)
+		return false;
 	while (len--) {
 		c = *s, d = *t;
 		if (islowerByte(c))
@@ -435,10 +419,10 @@ eb_bool memEqualCI(const char *s, const char *t, int len)
 		if (islowerByte(d))
 			d = toupper(d);
 		if (c != d)
-			return eb_false;
+			return false;
 		++s, ++t;
 	}
-	return eb_true;
+	return true;
 }				/* memEqualCI */
 
 char *strstrCI(const char *base, const char *search)
@@ -452,23 +436,41 @@ char *strstrCI(const char *base, const char *search)
 	return 0;
 }				/* strstrCI */
 
-eb_bool stringEqualCI(const char *s, const char *t)
+bool stringEqual(const char *s, const char *t)
+{
+/* check equality of strings with handling of null pointers */
+	if (s == t)
+		return true;
+	if (!s || !t)
+		return false;
+	if (strcmp(s, t))
+		return false;
+	return true;
+}				/* stringEqual */
+
+bool stringEqualCI(const char *s, const char *t)
 {
 	char c, d;
+/* if two pointers are equal we can return */
+	if (s == t)
+		return true;
+/* if one is NULL then the strings can't be equal */
+	if (!s || !t)
+		return false;
 	while ((c = *s) && (d = *t)) {
 		if (islowerByte(c))
 			c = toupper(c);
 		if (islowerByte(d))
 			d = toupper(d);
 		if (c != d)
-			return eb_false;
+			return false;
 		++s, ++t;
 	}
 	if (*s)
-		return eb_false;
+		return false;
 	if (*t)
-		return eb_false;
-	return eb_true;
+		return false;
+	return true;
 }				/* stringEqualCI */
 
 int stringInList(const char *const *list, const char *s)
@@ -506,7 +508,7 @@ int charInList(const char *list, char c)
 	char *s;
 	if (!list)
 		i_printfExit(MSG_NullCharInList);
-	s = strchr(list, c);
+	s = (char *)strchr(list, c);
 	if (!s)
 		return -1;
 	return s - list;
@@ -514,7 +516,7 @@ int charInList(const char *list, char c)
 
 /* In an empty list, next and prev point back to the list, not to 0. */
 /* We also allow zero. */
-eb_bool listIsEmpty(const struct listHead * l)
+bool listIsEmpty(const struct listHead * l)
 {
 	return l->next == l || l->next == 0;
 }				/* listIsEmpty */
@@ -526,37 +528,37 @@ void initList(struct listHead *l)
 
 void delFromList(void *x)
 {
-	struct listHead *xh = x;
+	struct listHead *xh = (struct listHead *)x;
 	((struct listHead *)xh->next)->prev = xh->prev;
 	((struct listHead *)xh->prev)->next = xh->next;
 }				/* delFromList */
 
 void addToListFront(struct listHead *l, void *x)
 {
-	struct listHead *xh = x;
+	struct listHead *xh = (struct listHead *)x;
 	xh->next = l->next;
 	xh->prev = l;
-	l->next = x;
-	((struct listHead *)xh->next)->prev = x;
+	l->next = (struct listHead *)x;
+	((struct listHead *)xh->next)->prev = (struct listHead *)x;
 }				/* addToListFront */
 
 void addToListBack(struct listHead *l, void *x)
 {
-	struct listHead *xh = x;
+	struct listHead *xh = (struct listHead *)x;
 	xh->prev = l->prev;
 	xh->next = l;
-	l->prev = x;
-	((struct listHead *)xh->prev)->next = x;
+	l->prev = (struct listHead *)x;
+	((struct listHead *)xh->prev)->next = (struct listHead *)x;
 }				/* addToListBack */
 
 void addAtPosition(void *p, void *x)
 {
-	struct listHead *xh = x;
-	struct listHead *ph = p;
+	struct listHead *xh = (struct listHead *)x;
+	struct listHead *ph = (struct listHead *)p;
 	xh->prev = p;
 	xh->next = ph->next;
-	ph->next = x;
-	((struct listHead *)xh->next)->prev = x;
+	ph->next = (struct listHead *)x;
+	((struct listHead *)xh->next)->prev = (struct listHead *)x;
 }				/* addAtPosition */
 
 void freeList(struct listHead *l)
@@ -569,14 +571,14 @@ void freeList(struct listHead *l)
 }				/* freeList */
 
 /* like isalnumByte, but allows _ and - */
-eb_bool isA(char c)
+bool isA(char c)
 {
 	if (isalnumByte(c))
-		return eb_true;
+		return true;
 	return (c == '_' || c == '-');
 }				/* isA */
 
-eb_bool isquote(char c)
+bool isquote(char c)
 {
 	return c == '"' || c == '\'';
 }				/* isquote */
@@ -657,7 +659,7 @@ pst clonePstring(pst s)
 	if (!s)
 		return s;
 	len = pstLength(s);
-	t = allocMem(len);
+	t = (pst) allocMem(len);
 	memcpy(t, s, len);
 	return t;
 }				/* clonePstring */
@@ -676,25 +678,25 @@ void copyPstring(pst s, const pst t)
  * This solves an outstanding issue, and it is needed for forthcoming
  * functionality, such as edpager.
  */
-eb_bool fdIntoMemory(int fd, char **data, int *len)
+bool fdIntoMemory(int fd, char **data, int *len)
 {
 	int length, n;
-	const int CHUNKSIZE = 8192;
+	const int blocksize = 8192;
 	char *chunk, *buf;
 
-	chunk = allocZeroMem(CHUNKSIZE);
+	chunk = allocZeroString(blocksize);
 	buf = initString(&length);
 
 	n = 0;
 	do {
-		n = read(fd, chunk, CHUNKSIZE);
+		n = read(fd, chunk, blocksize);
 		if (n < 0) {
 			nzFree(buf);
 			nzFree(chunk);
-			*data = EMPTYSTRING;
+			*data = emptyString;
 			*len = 0;
 			setError(MSG_NoRead, "file descriptor");
-			return eb_false;
+			return false;
 		}
 
 		if (n > 0)
@@ -702,29 +704,29 @@ eb_bool fdIntoMemory(int fd, char **data, int *len)
 	} while (n != 0);
 
 	nzFree(chunk);
-	buf = reallocMem(buf, length + 2);
+	buf = reallocString(buf, length + 2);
 	*data = buf;
 	*len = length;
-	return eb_true;
+	return true;
 }				/* fdIntoMemory */
 
-eb_bool fileIntoMemory(const char *filename, char **data, int *len)
+bool fileIntoMemory(const char *filename, char **data, int *len)
 {
 	int fh;
-	char ftype = fileTypeByName(filename, eb_false);
-	eb_bool ret;
+	char ftype = fileTypeByName(filename, false);
+	bool ret;
 	if (ftype && ftype != 'f') {
 		setError(MSG_RegularFile, filename);
-		return eb_false;
+		return false;
 	}
 	fh = open(filename, O_RDONLY | O_BINARY);
 	if (fh < 0) {
 		setError(MSG_NoOpen, filename);
-		return eb_false;
+		return false;
 	}
 
 	ret = fdIntoMemory(fh, data, len);
-	if (ret == eb_false)
+	if (ret == false)
 		setError(MSG_NoRead2, filename);
 
 	close(fh);
@@ -732,22 +734,22 @@ eb_bool fileIntoMemory(const char *filename, char **data, int *len)
 }				/* fileIntoMemory */
 
 /* inverse of the above */
-eb_bool memoryOutToFile(const char *filename, const char *data, int len,
+bool memoryOutToFile(const char *filename, const char *data, int len,
 /* specify the error messages */
-			int msgcreate, int msgwrite)
+		     int msgcreate, int msgwrite)
 {
 	int fh = open(filename, O_CREAT | O_TRUNC | O_WRONLY | O_BINARY, 0666);
 	if (fh < 0) {
 		setError(msgcreate, filename, errno);
-		return eb_false;
+		return false;
 	}
 	if (write(fh, data, len) < len) {
 		setError(msgwrite, filename, errno);
 		close(fh);
-		return eb_false;
+		return false;
 	}
 	close(fh);
-	return eb_true;
+	return true;
 }				/* memoryOutToFile */
 
 /* shift string to upper, lower, or mixed case */
@@ -756,7 +758,7 @@ void caseShift(char *s, char action)
 {
 	char c;
 	int mc = 0;
-	eb_bool ws = eb_true;
+	bool ws = true;
 
 	for (; c = *s; ++s) {
 		if (action == 'u') {
@@ -785,10 +787,10 @@ void caseShift(char *s, char action)
 			} else
 				mc = 0;
 			*s = c;
-			ws = eb_false;
+			ws = false;
 			continue;
 		}
-		ws = eb_true, mc = 0;
+		ws = true, mc = 0;
 	}			/* loop */
 }				/* caseShift */
 
@@ -803,24 +805,35 @@ is ported to other operating systems.
  * I think this will work on Windows, not sure.
  * But the link feature is Unix specific. */
 
-char fileTypeByName(const char *name, eb_bool showlink)
+char fileTypeByName(const char *name, bool showlink)
 {
 	struct stat buf;
-	eb_bool islink = eb_false;
+	bool islink = false;
 	char c;
 	int mode;
+
+#ifdef DOSLIKE
+	if (stat(name, &buf)) {
+		setError(MSG_NoAccess, name);
+		return 0;
+	}
+	mode = buf.st_mode & S_IFMT;
+#else // !DOSLIKE
+
 	if (lstat(name, &buf)) {
 		setError(MSG_NoAccess, name);
 		return 0;
 	}
 	mode = buf.st_mode & S_IFMT;
 	if (mode == S_IFLNK) {	/* symbolic link */
-		islink = eb_true;
+		islink = true;
 /* If this fails, I'm guessing it's just a file. */
 		if (stat(name, &buf))
 			return (showlink ? 'F' : 0);
 		mode = buf.st_mode & S_IFMT;
 	}
+#endif // DOSLIKE y/n
+
 	c = 'f';
 	if (mode == S_IFDIR)
 		c = 'd';
@@ -897,8 +910,211 @@ time_t fileTimeByName(const char *name)
 	return buf.st_mtime;
 }				/* fileTimeByName */
 
-#ifndef DOSLIKE
+char *conciseSize(size_t n)
+{
+	static char buf[32];
+	if (n >= (1 << 30))
+		sprintf(buf, "%luG", (unsigned long)n >> 30);
+	else if (n >= (1 << 20))
+		sprintf(buf, "%luM", (unsigned long)n >> 20);
+	else if (n >= (1 << 10))
+		sprintf(buf, "%luK", (unsigned long)n >> 10);
+	else
+		sprintf(buf, "%lu", (unsigned long)n);
+	return buf;
+}				/* conciseSize */
 
+char *conciseTime(time_t t)
+{
+	static char buffer[20];
+	static const char *months[] = {
+		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
+		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
+	};
+	struct tm *tm = localtime(&t);
+	sprintf(buffer, "%s %2d %d %02d:%02d",
+		months[tm->tm_mon], tm->tm_mday, tm->tm_year + 1900,
+		tm->tm_hour, tm->tm_min);
+	return buffer;
+}				/* conciseTime */
+
+/* retain only characters l s t i k p m, for ls attributes */
+bool lsattrChars(const char *buf, char *dest)
+{
+	bool rc = true;
+	const char *s;
+	char c, *t;
+#ifdef DOSLIKE
+	static const char ok_chars[] = "lst";
+#else
+	static const char ok_chars[] = "lstikpmy";
+#endif
+	char used[26];
+	memset(used, 0, sizeof(used));
+	t = dest;
+	for (s = buf; (c = *s); ++s) {
+		if (isspaceByte(c))
+			continue;
+		if (!strchr(ok_chars, c)) {
+			rc = false;
+			continue;
+		}
+		if (used[c - 'a'])
+			continue;
+		used[c - 'a'] = 1;
+		*t++ = c;
+	}
+	*t = 0;
+	return rc;
+}				/* lsattrChars */
+
+/* expand the ls attributes for a file into a static string. */
+/* This assumes user/group names will not be too long. */
+char *lsattr(const char *path, const char *flags)
+{
+	static char buf[200 + ABSPATH];
+	char p[40];
+	struct stat st, lst;
+	struct passwd *pwbuf;
+	struct group *grpbuf;
+	char *s;
+	int l, modebits;
+	bool sympath = false;	// user is requesting symlink path
+	bool statyes = true;	// stat call succeeds
+	char newpath[ABSPATH];
+
+	buf[0] = 0;
+
+	if (!path || !path[0] || !flags || !flags[0])
+		return buf;
+
+	if (strchr(flags, 'y'))
+		sympath = true;
+
+/* we already glommed onto this file through the directory listing;
+ * it ought to be there, except for broken symlink. */
+	if (stat(path, &st)) {
+		statyes = false;
+		if (!sympath)
+			return buf;
+	}
+
+	while (*flags) {
+		if (buf[0])
+			strcat(buf, " ");
+
+/* exception when asking for information on a broken symlink */
+		if (!statyes && *flags != 'y') {
+			strcat(buf, "?");
+			++flags;
+			continue;
+		}
+
+		switch (*flags) {
+		case 't':
+			strcat(buf, conciseTime(st.st_mtime));
+			break;
+		case 'l':
+			sprintf(p, "%lld", (long long)st.st_size);
+#ifndef DOSLIKE
+p:
+#endif // #ifndef DOSLIKE
+			strcat(buf, p);
+			break;
+		case 's':
+			strcat(buf, conciseSize(st.st_size));
+			break;
+#ifndef DOSLIKE
+/* not sure any of these work under windows */
+
+		case 'i':
+			sprintf(p, "%lu", (unsigned long)st.st_ino);
+			goto p;
+		case 'k':
+			sprintf(p, "%lu", (unsigned long)st.st_nlink);
+			goto p;
+		case 'm':
+			strcpy(p, "-");
+			if (st.st_rdev)
+				sprintf(p, "%d/%d",
+					(int)(st.st_rdev >> 8),
+					(int)(st.st_rdev & 0xff));
+			goto p;
+		case 'p':
+			s = buf + strlen(buf);
+			pwbuf = getpwuid(st.st_uid);
+			if (pwbuf) {
+				l = strlen(pwbuf->pw_name);
+				if (l > 20)
+					l = 20;
+				strncpy(s, pwbuf->pw_name, l);
+				s[l] = 0;
+			} else
+				sprintf(s, "%d", st.st_uid);
+			s += strlen(s);
+			*s++ = ' ';
+			grpbuf = getgrgid(st.st_gid);
+			if (grpbuf) {
+				l = strlen(grpbuf->gr_name);
+				if (l > 20)
+					l = 20;
+				strncpy(s, grpbuf->gr_name, l);
+				s[l] = 0;
+			} else
+				sprintf(s, "%d", st.st_gid);
+			s += strlen(s);
+			*s++ = ' ';
+			modebits = st.st_mode;
+			modebits &= 07777;
+			if (modebits & 07000)
+				*s++ = '0' + (modebits >> 9);
+			modebits &= 0777;
+			*s++ = '0' + (modebits >> 6);
+			modebits &= 077;
+			*s++ = '0' + (modebits >> 3);
+			modebits &= 7;
+			*s++ = '0' + modebits;
+			*s = 0;
+			break;
+
+		case 'y':
+			if (lstat(path, &lst)) {
+/* Wow, this call should always succeed */
+				strcat(buf, "?");
+				break;
+			}
+			if ((lst.st_mode & S_IFMT) != S_IFLNK) {
+				strcat(buf, "-");
+				break;
+			}
+/* yes it's a link, read the path */
+			l = readlink(path, newpath, sizeof(newpath));
+			if (l <= 0)
+				strcat(buf, "?");
+			else {
+				s = buf + strlen(buf);
+				strncpy(s, newpath, l);
+				s[l] = 0;
+			}
+			break;
+
+#endif
+
+		}
+
+		++flags;
+	}
+
+	return buf;
+}				/* lsattr */
+
+#ifdef DOSLIKE
+void ttySaveSettings(void)
+{
+	// TODO: Anything needed here for WIN32?
+	isInteractive = _isatty(0);
+}
+#else // !#ifdef DOSLIKE
 static struct termios savettybuf;
 void ttySaveSettings(void)
 {
@@ -922,7 +1138,7 @@ static void ttyRestoreSettings(void)
  * min>0 time=0:  block until min chars are received.
  * min=0 time>0:  return 1 char, or 0 if the timer expires.
  * min=0 time=0:  nonblocking, return whatever chars have been received. */
-static void ttyRaw(int charcount, int timeout, eb_bool isecho)
+static void ttyRaw(int charcount, int timeout, bool isecho)
 {
 	struct termios buf = savettybuf;	/* structure copy */
 	buf.c_cc[VMIN] = charcount;
@@ -938,7 +1154,7 @@ int getche(void)
 {
 	char c;
 	fflush(stdout);
-	ttyRaw(1, 0, eb_true);
+	ttyRaw(1, 0, true);
 	read(0, &c, 1);
 	ttyRestoreSettings();
 	return c;
@@ -948,18 +1164,18 @@ int getch(void)
 {
 	char c;
 	fflush(stdout);
-	ttyRaw(1, 0, eb_false);
+	ttyRaw(1, 0, false);
 	read(0, &c, 1);
 	ttyRestoreSettings();
 	return c;
 }				/* getche */
 
-#endif
+#endif // #ifdef DOSLIKE y/n
 
 char getLetter(const char *s)
 {
 	char c;
-	while (eb_true) {
+	while (true) {
 		c = getch();
 		if (strchr(s, c))
 			break;
@@ -970,131 +1186,137 @@ char getLetter(const char *s)
 	return c;
 }				/* getLetter */
 
-/* loop through the files in a directory */
-/* Hides the differences between DOS, Unix, and NT. */
-static eb_bool dirstart = eb_true;
-
-char *nextScanFile(const char *base)
+/* Parameters: message, default file name, must this file be new,
+ * and can we except an input of white space,
+ * that being converted to a single space. */
+char *getFileName(int msg, const char *defname, bool isnew, bool ws)
 {
-	char *s;
-#ifdef DOSLIKE
-	static char global[] = "/*.*";
-	eb_bool rc;
-	short len;
+	static char buf[ABSPATH];
+	static char spacename[] = " ";
+	int l;
 	char *p;
-	eb_bool allocate = eb_false;
-#ifdef MSDOS
-	static struct _find_t dta;
-#else
-	static struct _finddata_t dta;
-	static int handle;
-#endif
-#else
-	struct dirent *de;
-	static DIR *df;
-#endif
+	bool allspace;
 
-#ifdef DOSLIKE
-	if (dirstart) {
-		if (base) {
-			len = strlen(base) - 1;
-			p = allocMem(len + 6);
-			strcpy(p, base);
-			allocate = eb_true;
-			if (p[len] == '/' || p[len] == '\\')
-				p[len] = 0;
-			strcat(p, global);
+	while (true) {
+		i_printf(msg);
+		if (defname)
+			printf("[%s] ", defname);
+		fflush(stdout);
+		if (!fgets(buf, sizeof(buf), stdin))
+			exit(0);
+		allspace = false;
+		for (p = buf; isspaceByte(*p); ++p)
+			if (*p == ' ')
+				allspace = true;
+		l = strlen(p);
+		while (l && isspaceByte(p[l - 1]))
+			--l;
+		p[l] = 0;
+		if (!l) {
+			if (ws & allspace)
+				return spacename;
+			if (!defname)
+				continue;
+/* make a copy just to be safe */
+			l = strlen(defname);
+			if (l >= ABSPATH)
+				l = ABSPATH - 1;
+			strncpy(buf, defname, l);
+			buf[l] = 0;
+			p = buf;
 		} else
-			p = global +1;
-#ifdef MSDOS
-		rc = _dos_findfirst(p, (showHiddenFiles ? 077 : 073), &dta);
-#else
-		rc = eb_false;
-		handle = _findfirst(p, &dta);
-		if (handle < 0)
-			rc = eb_true;
-#endif
-		if (allocate)
-			nzFree(p);
+			defname = 0;
+		if (isnew && fileTypeByName(p, false)) {
+			i_printf(MSG_FileExists, p);
+			defname = 0;
+			continue;
+		}
+		return p;
 	}
-#else
+}				/* getFileName */
+
+/* Protect a filename from expansion by the shell */
+static const char shellmeta[] = "\\\n\t |&;<>(){}#'\"~$*?";
+int shellProtectLength(const char *s)
+{
+	int l = 0;
+	while (*s) {
+		if (strchr(shellmeta, *s))
+			++l;
+		++l, ++s;
+	}
+	return l;
+}				/* shellProtectLength */
+
+void shellProtect(char *t, const char *s)
+{
+	while (*s) {
+		if (strchr(shellmeta, *s))
+			*t++ = '\\';
+		*t++ = *s++;
+	}
+}				/* shellProtect */
+
+/* loop through the files in a directory */
+const char *nextScanFile(const char *base)
+{
+	static DIR *df;
+	struct dirent *de;
+	const char *s;
+
 	if (!df) {
 		if (!base)
 			base = ".";
 		df = opendir(base);
+/* directory could be unreadable */
 		if (!df) {
 			i_puts(MSG_NoDirNoList);
 			return 0;
 		}
 	}
-#endif
 
-#ifdef DOSLIKE
-	while (eb_true) {
-/* read the next file */
-		if (!dirStart) {
-#ifdef MSDOS
-			rc = _dos_findnext(&dta);
-#else
-			rc = _findnext(handle, &dta);
-#endif
-			dirstart = eb_false;
-		}
-		if (rc)
-			break;
-/* extract the base name */
-		s = strrchr(dta.name, '/');
-		s = s ? s + 1 : dta.name;
-/* weed out unwanted directories */
-		if (stringEqual(s, "."))
-			continue;
-		if (stringEqual(s, ".."))
-			continue;
-		return s;
-	}			/* end loop over files in directory */
-#else
 	while (de = readdir(df)) {
-		if (de->d_ino == 0)
-			continue;
-		if (de->d_name[0] == '.') {
+		s = de->d_name;
+		if (s[0] == '.') {
+			if (stringEqual(s, "."))
+				continue;
+			if (stringEqual(s, ".."))
+				continue;
 			if (!showHiddenFiles)
 				continue;
-			if (de->d_name[1] == 0)
-				continue;
-			if (de->d_name[1] == '.' && de->d_name[2] == 0)
-				continue;
 		}
-		return de->d_name;
+		return s;
 	}			/* end loop over files in directory */
+
 	closedir(df);
 	df = 0;
-#endif
-
 	return 0;
 }				/* nextScanFile */
 
-static int qscmp(const void *s, const void *t)
+/* compare routine for quicksort */
+static int dircmp(const void *s, const void *t)
 {
-	return strcmp(((struct lineMap *)s)->text, ((struct lineMap *)t)->text);
-}				/* qscmp */
+	return strcoll((const char *)((const struct lineMap *)s)->text,
+		       (const char *)((const struct lineMap *)t)->text);
+}				/* dircmp */
 
-eb_bool sortedDirList(const char *dir, struct lineMap **map_p, int *count_p)
+bool sortedDirList(const char *dir, struct lineMap **map_p, int *count_p)
 {
-	char *f;
+	const char *f;
 	int linecount = 0, cap;
 	struct lineMap *t, *map;
 
 	cap = 128;
-	map = t = allocZeroMem(cap * LMSIZE);
+	map = t = (struct lineMap *)allocZeroMem(cap * LMSIZE);
 
 	while (f = nextScanFile(dir)) {
 		if (linecount == cap) {
 			cap *= 2;
-			map = reallocMem(map, cap * LMSIZE);
+			map = (struct lineMap *)reallocMem(map, cap * LMSIZE);
 			t = map + linecount;
 		}
 /* leave room for @ / newline */
-		t->text = allocMem(strlen(f) + 3);
+		t->text = (pst) allocMem(strlen(f) + 3);
 		strcpy((char *)t->text, f);
 		t->ds1 = t->ds2 = 0;
 		++t, ++linecount;
@@ -1104,11 +1326,12 @@ eb_bool sortedDirList(const char *dir, struct lineMap **map_p, int *count_p)
 	*map_p = map;
 
 	if (!linecount)
-		return eb_true;
+		return true;
 
-	qsort(map, linecount, LMSIZE, qscmp);
+/* sort the entries */
+	qsort(map, linecount, LMSIZE, dircmp);
 
-	return eb_true;
+	return true;
 }				/* sortedDirList */
 
 /* Expand environment variables, then wild cards.
@@ -1117,195 +1340,215 @@ eb_bool sortedDirList(const char *dir, struct lineMap **map_p, int *count_p)
  * Neither the original line nore the new line is allocated.
  * They are static char buffers that are just plain long enough. */
 
-eb_bool envFile(const char *line, const char **expanded)
+static bool envExpand(const char *line, const char **expanded)
 {
-	static char line1[MAXTTYLINE];
-	static char line2[MAXTTYLINE];
-	const char *s, *value, *basedir;
-	char *t, *dollar, *cut, *file;
-	char c;
-	eb_bool cc, badBrackets;
-	int filecount;
-	char re[MAXRE + 20];
-	const char *re_error;
-	int re_offset, re_count, re_vector[3];
+	const char *s;
+	char *t;
+	const char *v;		/* result of getenv call */
+	bool inbrace;		/* ${foo} */
+	struct passwd *pw;
+	const char *udir;	/* user directory */
+	int l;
+	static char varline[ABSPATH];
+	char var1[40];
 
-	pcre *re_cc;
-
-/* ` supresses this stuff */
-	if (*line == '`') {
-		*expanded = line + 1;
-		return eb_true;
-	}
-
-/* quick check, nothing to do */
-	if (!strpbrk(line, "$[*?") && line[0] != '~') {
+/* quick check */
+	if (line[0] != '~' && !strchr(line, '$')) {
 		*expanded = line;
-		return eb_true;
+		return true;
 	}
 
-/* first the env variables */
+/* ok, need to crunch along */
+	t = varline;
 	s = line;
-	t = line1;
-	dollar = 0;
-	while (eb_true) {
-		if (t >= line1 + sizeof(line1))
-			goto longvar;
-		c = *s;
-		if (c == '~' && s == line && (s[1] == '/' || s[1] == 0)) {
-			if (strlen(home) >= sizeof(line1) - 1)
-				goto longvar;
-			strcpy(t, home);
-			t = t + strlen(t);
+
+	if (line[0] != '~')
+		goto dollars;
+
+	l = 0;
+	for (s = line + 1; isalnum(*s) || *s == '_'; ++s)
+		++l;
+	if (l >= sizeof(var1) || isdigit(line[1]) || *s && *s != '/') {
+/* invalid syntax, put things back */
+		s = line;
+		goto dollars;
+	}
+
+	udir = 0;
+	strncpy(var1, line + 1, l);
+	var1[l] = 0;
+#ifndef DOSLIKE
+	if (l) {
+		pw = getpwnam(var1);
+		if (!pw) {
+			setError(MSG_NoTilde, var1);
+			return false;
+		}
+		if (pw->pw_dir && *pw->pw_dir)
+			udir = pw->pw_dir;
+	} else
+#endif
+		udir = home;
+	if (!udir) {
+		s = line;
+		goto dollars;
+	}
+	l = strlen(udir);
+	if (l >= sizeof(varline))
+		goto longline;
+	strcpy(varline, udir);
+	t = varline + l;
+
+dollars:
+	for (; *s; ++s) {
+		if (t - varline == ABSPATH - 1) {
+longline:
+			setError(MSG_ShellLineLong);
+			return false;
+		}
+		if (*s == '\\' && s[1] == '$') {
+/* this $ is escaped */
 			++s;
+appendchar:
+			*t++ = *s;
 			continue;
 		}
-		if (dollar && !isalnumByte(c) && c != '_') {
-			*t = 0;
-			value = getenv(dollar + 1);
-			if (!value) {
-				setError(MSG_NoEnvVar, dollar + 1);
-				return eb_false;
-			}
-			if (dollar + strlen(value) >= line1 + sizeof(line1) - 1)
-				goto longvar;
-			strcpy(dollar, value);
-			t = dollar + strlen(dollar);
-			dollar = 0;
-		}
-		if (c == '$' && (s[1] == '_' || isalphaByte(s[1])))
-			dollar = t;
-		*t++ = c;
-		if (!c)
-			break;
-		++s;
-	}
+		if (*s != '$')
+			goto appendchar;
 
-/* Wildcard expansion, but somewhat limited.
- * The directory has to be hard coded;
- * we only expand stars in the filename. */
-
-	cut = dollar = 0;
-	for (t = line1; *t; ++t) {
-		c = *t;
-		if (c == '/')
-			cut = t;
-		if (c == '[' || c == '*' || c == '?')
-			dollar = t;
+/* this is $, see if it is $var or ${var} */
+		inbrace = false;
+		v = s + 1;
+		if (*v == '{')
+			inbrace = true, ++v;
+		if (!isalphaByte(*v) && *v != '_')
+			goto appendchar;
+		l = 0;
+		while (isalnumByte(*v) || *v == '_') {
+			if (l == sizeof(var1) - 1)
+				goto longline;
+			var1[l++] = *v++;
+		}
+		var1[l] = 0;
+		if (inbrace) {
+			if (*v != '}')
+				goto appendchar;
+			++v;
+		}
+		s = v - 1;
+		v = getenv(var1);
+		if (!v) {
+			setError(MSG_NoEnvVar, var1);
+			return false;
+		}
+		l = strlen(v);
+		if (t - varline + l >= ABSPATH)
+			goto longline;
+		strcpy(t, v);
+		t += l;
 	}
-	if (!dollar) {		/* nothing meta */
-		*expanded = line1;
-		return eb_true;
-	}
-	if (cut && dollar < cut) {
-		setError(MSG_EarlyExpand);
-		return eb_false;
-	}
-
-/* Make sure its a directory, and build a perl regexp for the pattern. */
-	if (cut) {
-		*cut = 0;
-		if (cut > line1 && fileTypeByName(line1, eb_false) != 'd') {
-			setError(MSG_NoAccessDir, line1);
-			return eb_false;
-		}
-	}
-
-	s = cut ? cut + 1 : line1;
-	t = re;
-	*t++ = '^';
-	cc = badBrackets = eb_false;
-	while (c = *s) {
-		if (t >= re + sizeof(re) - 3) {
-			setError(MSG_ShellPatternLong);
-			return eb_false;
-		}
-		if (c == '\\') {
-			setError(MSG_ExpandBackslash);
-			return eb_false;
-		}
-/* things we need to escape */
-		if (strchr("().+|", c))
-			*t++ = '\\';
-		if (c == '?')
-			c = '.';
-		if (c == '*')
-			*t++ = '.';
-		*t++ = c;
-		if (c == '[') {
-			if (cc)
-				badBrackets = eb_true;
-			cc = eb_true;
-		}
-		if (c == ']') {
-			if (!cc)
-				badBrackets = eb_true;
-			cc = eb_false;
-		}
-		++s;
-	}			/* loop over shell pattern */
-	*t++ = '$';
 	*t = 0;
-	if (badBrackets | cc) {
-		setError(MSG_ShellSyntax);
-		return eb_false;
-	}
 
-	debugPrint(7, "shell regexp %s", re);
-	re_cc = pcre_compile(re, 0, &re_error, &re_offset, 0);
-	if (!re_cc) {
-		setError(MSG_ShellCompile, re_error);
-		return eb_false;
-	}
+	*expanded = varline;
+	return true;
+}				/* envExpand */
 
-	filecount = 0;
-	cc = eb_false;		/* long flag */
-	basedir = 0;
-	if (cut) {
-		if (cut == line1)
-			basedir = "/";
-		else
-			basedir = line1;
-	}
-	while (file = nextScanFile(basedir)) {
-		if (filecount > 1)
-			continue;
-		re_count =
-		    pcre_exec(re_cc, 0, file, strlen(file), 0, 0, re_vector, 3);
-		if (re_count < -1) {
-			pcre_free(re_cc);
-			setError(MSG_ShellExpand);
-			return eb_false;
+bool envFile(const char *line, const char **expanded)
+{
+	static char line2[ABSPATH];
+	const char *varline;
+	const char *s;
+	char *t;
+#ifndef DOSLIKE
+	glob_t g;
+#endif // #ifndef DOSLIKE
+	int rc, flags;
+
+/* ` disables this stuff */
+/* but `` is a literal ` */
+	if (line[0] == '`') {
+		if (line[1] != '`') {
+			*expanded = line + 1;
+			return true;
 		}
-		if (re_count < 0)
-			continue;
-		++filecount;
-		if (filecount > 1)
-			continue;
-		if ((cut ? strlen(line1) : 0) + strlen(file) >=
-		    sizeof(line2) - 2)
-			cc = eb_true;
-		else if (cut)
-			sprintf(line2, "%s/%s", line1, file);
-		else
-			strcpy(line2, file);
+		++line;
 	}
-	pcre_free(re_cc);
-	if (filecount != 1) {
-		setError((filecount > 0) + MSG_ShellNoMatch);
-		return eb_false;
-	}
-	if (cc)
-		goto longvar;
 
+	if (!envExpand(line, &varline))
+		return false;
+
+#ifdef DOSLIKE
+	*expanded = varline;
+	return true;		// TODO: WIN32: Expand like glob...
+#else // !#ifdef DOSLIKE
+
+/* expanded the environment variables, if any, now time to glob */
+	flags = GLOB_NOSORT;
+	rc = glob(varline, flags, NULL, &g);
+
+	if (rc == GLOB_NOMATCH) {
+/* unescape the metas */
+		t = line2;
+		for (s = varline; *s; ++s) {
+			if (*s == '\\' && s[1] && strchr("*?[", s[1]))
+				++s;
+			*t++ = *s;
+		}
+		*t = 0;
+		*expanded = line2;
+		return true;
+	}
+
+	if (rc) {
+/* some other syntax error, whereup we can't expand. */
+		setError(MSG_ShellExpand);
+		globfree(&g);
+		return false;
+	}
+
+	if (g.gl_pathc != 1) {
+		setError(MSG_ShellManyMatch);
+		globfree(&g);
+		return false;
+	}
+
+/* looks good, if it isn't too long */
+	s = g.gl_pathv[0];
+	if (strlen(s) >= sizeof(line2)) {
+		setError(MSG_ShellLineLong);
+		globfree(&g);
+		return false;
+	}
+
+	strcpy(line2, s);
+	globfree(&g);
 	*expanded = line2;
-	return eb_true;
+	return true;
+#endif // #ifdef DOSLIKE y/n
 
-longvar:
-	setError(MSG_ShellLineLong);
-	return eb_false;
 }				/* envFile */
 
+/* Call the above routine if filename contains a  slash,
+ * or prepend the download directory if it does not.
+ * If there is no download directory then always expand as above. */
+bool envFileDown(const char *line, const char **expanded)
+{
+	static char line2[MAXTTYLINE];
+
+	if (!downDir || strchr(line, '/'))
+/* we don't necessarily expect there to be a file here */
+		return envFile(line, expanded);
+
+	if (strlen(downDir) + strlen(line) >= sizeof(line2) - 1) {
+		setError(MSG_ShellLineLong);
+		return false;
+	}
+
+	sprintf(line2, "%s/%s", downDir, line);
+	*expanded = line2;
+	return true;
+}				/* envFileDown */
+
 FILE *efopen(const char *name, const char *mode)
 {
 	FILE *f;
@@ -1363,60 +1606,26 @@ void appendFileNF(const char *filename, const char *msg)
 	fclose(f);
 }				/* appendFileNF */
 
-/*********************************************************************
-Routines to convert between names and IP addresses.
-This is ipv4; need to write similar for ipv6.
-*********************************************************************/
-
-IP32bit tcp_name_ip(const char *name)
+/* Wrapper around system(). */
+int eb_system(const char *cmd, bool print_on_success)
 {
-	struct hostent *hp;
-	IP32bit *ip;
-
-	hp = gethostbyname(name);
-	if (!hp)
-		return NULL_IP;
-	ip = (IP32bit *) * (hp->h_addr_list);
-	if (!ip)
-		return NULL_IP;
-	return *ip;
-}				/* tcp_name_ip */
-
-char *tcp_ip_dots(IP32bit ip)
-{
-	return inet_ntoa(*(struct in_addr *)&ip);
-}				/* tcp_ip_dots */
-
-int tcp_isDots(const char *s)
-{
-	const char *t;
-	char c;
-	int nd = 0;		/* number of dots */
-	if (!s)
-		return 0;
-	for (t = s; (c = *t); ++t) {
-		if (c == '.') {
-			++nd;
-			if (t == s || !t[1])
-				return 0;
-			if (t[-1] == '.' || t[1] == '.')
-				return 0;
-			continue;
-		}
-		if (!isdigit(c))
-			return 0;
-	}
-	return (nd == 3);
-}				/* tcp_isDots */
-
-IP32bit tcp_dots_ip(const char *s)
-{
-	struct in_addr a;
-/* this for SCO unix */
-#ifdef SCO
-	inet_aton(s, &a);
+	int system_ret = 0;
+#ifdef DOSLIKE
+	system_ret = system(cmd);
 #else
-	*(IP32bit *) & a = inet_addr(s);
+/* Ignoring of SIGPIPE propagates across fork-exec. */
+/* So stop ignoring it for the duration of system(). */
+	signal(SIGPIPE, SIG_DFL);
+	system_ret = system(cmd);
+	signal(SIGPIPE, SIG_IGN);
 #endif
-	return *(IP32bit *) & a;
-}				/* tcp_dots_ip */
+
+	if (system_ret == 0) {
+		if (print_on_success)
+			i_puts(MSG_OK);
+	} else {
+		i_printf(MSG_SystemCmdFail, system_ret);
+		nl();
+	}
+	return system_ret;
+}				/* eb_system */
