@@ -1,61 +1,62 @@
 /* url.c
- * (c) 2002 Mikulas Patocka
- * This file is part of the Links project, released under GPL.
- *
- * Modified by Karl Dahlke for integration with edbrowse.
+ * Separate a url into pieces, turn a relative file into an absolute url.
+ * This file is part of the edbrowse project, released under GPL.
  */
 
 #include "eb.h"
 
-struct {
-	char *prot;
+struct PROTOCOL {
+	const char prot[12];
 	int port;
-	eb_bool free_syntax;
-	eb_bool need_slashes;
-	eb_bool need_slash_after_host;
+	bool free_syntax;
+	bool need_slashes;
+	bool need_slash_after_host;
 } protocols[] = {
 	{
-	"file", 0, eb_true, eb_true, eb_false}, {
-	"http", 80, eb_false, eb_true, eb_true}, {
-	"https", 443, eb_false, eb_true, eb_true}, {
-	"pop3", 110, eb_false, eb_true, eb_true}, {
-	"pop3s", 995, eb_false, eb_true, eb_true}, {
-	"smtp", 25, eb_false, eb_true, eb_true}, {
-	"submission", 587, eb_false, eb_true, eb_true}, {
-	"smtps", 465, eb_false, eb_true, eb_true}, {
-	"proxy", 3128, eb_false, eb_true, eb_true}, {
-	"ftp", 21, eb_false, eb_true, eb_true}, {
-	"sftp", 22, eb_false, eb_true, eb_true}, {
-	"ftps", 990, eb_false, eb_true, eb_true}, {
-	"tftp", 69, eb_false, eb_true, eb_true}, {
-	"rtsp", 554, eb_false, eb_true, eb_true}, {
-	"pnm", 7070, eb_false, eb_true, eb_true}, {
-	"finger", 79, eb_false, eb_true, eb_true}, {
-	"smb", 139, eb_false, eb_true, eb_true}, {
-	"mailto", 0, eb_false, eb_false, eb_false}, {
-	"telnet", 23, eb_false, eb_false, eb_false}, {
-	"tn3270", 0, eb_false, eb_false, eb_false}, {
-	"javascript", 0, eb_true, eb_false, eb_false}, {
-	"git", 0, eb_false, eb_false, eb_false}, {
-	"svn", 0, eb_false, eb_false, eb_false}, {
-	"gopher", 70, eb_false, eb_false, eb_false}, {
-	"magnet", 0, eb_false, eb_false, eb_false}, {
-	"irc", 0, eb_false, eb_false, eb_false}, {
-	NULL, 0}
-};
+	"file", 0, true, true, false}, {
+	"http", 80, false, true, true}, {
+	"https", 443, false, true, true}, {
+	"pop3", 110, false, true, true}, {
+	"pop3s", 995, false, true, true}, {
+	"imap", 220, false, true, true}, {
+	"imaps", 993, false, true, true}, {
+	"smtp", 25, false, true, true}, {
+	"submission", 587, false, true, true}, {
+	"smtps", 465, false, true, true}, {
+	"proxy", 3128, false, true, true}, {
+	"ftp", 21, false, true, true}, {
+	"sftp", 22, false, true, true}, {
+	"scp", 22, false, true, true}, {
+	"ftps", 990, false, true, true}, {
+	"tftp", 69, false, true, true}, {
+	"rtsp", 554, false, true, true}, {
+	"pnm", 7070, false, true, true}, {
+	"finger", 79, false, true, true}, {
+	"smb", 139, false, true, true}, {
+	"mailto", 0, false, false, false}, {
+	"telnet", 23, false, false, false}, {
+	"tn3270", 0, false, false, false}, {
+	"data", 0, true, false, false}, {
+	"javascript", 0, true, false, false}, {
+	"git", 0, false, false, false}, {
+	"svn", 0, false, false, false}, {
+	"gopher", 70, false, false, false}, {
+	"magnet", 0, false, false, false}, {
+	"irc", 0, false, false, false}, {
+"", 0},};
 
-static eb_bool free_syntax;
+static bool free_syntax;
 
 static int protocolByName(const char *p, int l)
 {
 	int i;
-	for (i = 0; protocols[i].prot; i++)
+	for (i = 0; protocols[i].prot[0]; i++)
 		if (memEqualCI(protocols[i].prot, p, l))
 			return i;
 	return -1;
 }				/* protocolByName */
 
-/* Unpercent the host component of a url.  Christ what a pain! */
+/* Unpercent the host component of a url, not the data component. */
 void unpercentURL(char *url)
 {
 	char c, *u, *w;
@@ -63,6 +64,8 @@ void unpercentURL(char *url)
 	u = w = url;
 	while (c = *u) {
 		++u;
+		if (c == '+')
+			c = ' ';
 		if (c == '%' && isxdigit(u[0]) && isxdigit(u[1])) {
 			c = fromHex(u[0], u[1]);
 			u += 2;
@@ -83,8 +86,131 @@ void unpercentURL(char *url)
 	strmove(w, u);
 }				/* unpercentURL */
 
+/* Unpercent an entire string. */
+void unpercentString(char *s)
+{
+	char c, *u, *w;
+	u = w = s;
+	while (c = *u) {
+		++u;
+		if (c == '+')
+			c = ' ';
+		if (c == '%' && isxdigit(u[0]) && isxdigit(u[1])) {
+			c = fromHex(u[0], u[1]);
+			u += 2;
+		}
+		if (!c)
+			c = ' ';	/* should never happen */
+		*w++ = c;
+	}
+	*w = 0;
+}				/* unpercentString */
+
+/*
+ * Function: percentURL
+ * Arguments:
+ ** start: pointer to start of input string
+  ** end: pointer to end of input string.
+ * Return value: A new string or NULL if memory allocation failed.
+ * This function copies its input to a dynamically-allocated buffer,
+ * while performing the following transformation.  Change backslash to slash,
+ * and percent-escape some of the reserved characters as per RFC3986.
+ * Some of the chars retain their reserved semantics and should not be changed.
+ * This is a friggin guess!
+ * All characters in the area between start and end, not including end,
+ * are copied or transformed.
+ * Get rid of :/   curl can't handle it.
+ * This function is used to sanitize user-supplied URLs.  */
+
+/* these punctuations are percentable, anywhere in a url */
+static const char percentable[] = "!*'()[]+$,";
+static char hexdigits[] = "0123456789abcdef";
+#define ESCAPED_CHAR_LENGTH 3
+
+char *percentURL(const char *start, const char *end)
+{
+	int bytes_to_alloc;
+	char *new_copy;
+	const char *in_pointer;
+	char *out_pointer;
+	const char *portloc;
+
+	if (!end)
+		end = start + strlen(start);
+	bytes_to_alloc = end - start + 1;
+	new_copy = NULL;
+	in_pointer = NULL;
+	out_pointer = NULL;
+	portloc = NULL;
+
+	for (in_pointer = start; in_pointer < end; in_pointer++)
+		if (*in_pointer <= ' ' || strchr(percentable, *in_pointer))
+			bytes_to_alloc += (ESCAPED_CHAR_LENGTH - 1);
+
+	new_copy = allocMem(bytes_to_alloc);
+	if (new_copy) {
+		char *frag, *params;
+		out_pointer = new_copy;
+		for (in_pointer = start; in_pointer < end; in_pointer++) {
+			if (*in_pointer == '\\')
+				*out_pointer++ = '/';
+			else if (*in_pointer <= ' ' ||
+				 strchr(percentable, *in_pointer)) {
+				*out_pointer++ = '%';
+				*out_pointer++ =
+				    hexdigits[(uchar) (*in_pointer & 0xf0) >>
+					      4];
+				*out_pointer++ =
+				    hexdigits[(*in_pointer & 0x0f)];
+			} else
+				*out_pointer++ = *in_pointer;
+		}
+		*out_pointer = '\0';
+/* excise #hash, required by some web servers */
+		frag = findHash(new_copy);
+		if (frag)
+			*frag = 0;
+
+		getPortLocURL(new_copy, &portloc, 0);
+		if (portloc && !isdigit(portloc[1])) {
+			const char *s = portloc + strcspn(portloc, "/?#\1");
+			strmove((char *)portloc, s);
+		}
+	}
+
+	return new_copy;
+}				/* percentURL */
+
+/* escape & < > for display on a web page */
+char *htmlEscape0(const char *s, bool do_and)
+{
+	char *t;
+	int l;
+	if (!s)
+		return 0;
+	if (!*s)
+		return emptyString;
+	t = initString(&l);
+	for (; *s; ++s) {
+		if (*s == '&' && do_and) {
+			stringAndString(&t, &l, "&amp;");
+			continue;
+		}
+		if (*s == '<') {
+			stringAndString(&t, &l, "&lt;");
+			continue;
+		}
+		if (*s == '>') {
+			stringAndString(&t, &l, "&gt;");
+			continue;
+		}
+		stringAndChar(&t, &l, *s);
+	}
+	return t;
+}				/* htmlEscape0 */
+
 /* Decide if it looks like a web url. */
-static eb_bool httpDefault(const char *url)
+static bool httpDefault(const char *url)
 {
 	static const char *const domainSuffix[] = {
 		"com", "biz", "info", "net", "org", "gov", "edu", "us", "uk",
@@ -110,14 +236,14 @@ static eb_bool httpDefault(const char *url)
 		if (*s == '.' && s[-1] != '.' && s[1] != '.')
 			++n, lastdot = s;
 	if (n < 2)
-		return eb_false;
+		return false;
 /* All digits, like an ip address, is ok. */
 	if (n == 3) {
 		for (s = url; s < end; ++s)
 			if (!isdigitByte(*s) && *s != '.')
 				break;
 		if (s == end)
-			return eb_true;
+			return true;
 	}
 /* Look for standard domain suffix */
 	++lastdot;
@@ -125,13 +251,20 @@ static eb_bool httpDefault(const char *url)
 	for (n = 0; domainSuffix[n]; ++n)
 		if (memEqualCI(lastdot, domainSuffix[n], len)
 		    && !domainSuffix[n][len])
-			return eb_true;
+			return true;
 /* www.anything.xx is ok */
 	if (len == 2 && memEqualCI(url, "www.", 4))
-		return eb_true;
-	return eb_false;
+		return true;
+	return false;
 }				/* httpDefault */
 
+/*********************************************************************
+From wikipedia url
+scheme://domain:port/path?query_string#fragment_id
+but I allow, at the end of this, control a followed by post data, with the
+understanding that there should not be query_string and post data simultaneously.
+*********************************************************************/
+
 static int parseURL(const char *url, const char **proto, int *prlen, const char **user, int *uslen, const char **pass, int *palen,	/* ftp protocol */
 		    const char **host, int *holen,
 		    const char **portloc, int *port,
@@ -166,7 +299,7 @@ static int parseURL(const char *url, const char **proto, int *prlen, const char
 		*dalen = 0;
 	if (post)
 		*post = NULL;
-	free_syntax = eb_false;
+	free_syntax = false;
 
 	if (!url)
 		return -1;
@@ -181,10 +314,9 @@ static int parseURL(const char *url, const char **proto, int *prlen, const char
 			++q;
 		if (*q == '/')
 			++q;
-		while (isspaceByte(*q))
-			++q;
+		skipWhite(&q);
 		if (!*q)
-			return eb_false;
+			return false;
 		a = protocolByName(url, p - url);
 	}
 	if (a >= 0) {
@@ -230,16 +362,17 @@ static int parseURL(const char *url, const char **proto, int *prlen, const char
 	}
 
 	if (a < 0)
-		return eb_false;
+		return false;
 
 	if (free_syntax = protocols[a].free_syntax) {
 		if (data)
 			*data = p;
 		if (dalen)
 			*dalen = strlen(p);
-		return eb_true;
+		return true;
 	}
 
+/* find the end of the domain */
 	q = p + strcspn(p, "@?#/\1");
 	if (*q == '@') {	/* user:password@host */
 		const char *pp = strchr(p, ':');
@@ -261,6 +394,7 @@ static int parseURL(const char *url, const char **proto, int *prlen, const char
 		p = q + 1;
 	}
 
+/* again, look for the end of the domain */
 	q = p + strcspn(p, ":?#/\1");
 	if (host)
 		*host = p;
@@ -299,28 +433,33 @@ static int parseURL(const char *url, const char **proto, int *prlen, const char
 		*dalen = q - p;
 	if (post)
 		*post = *q ? q + 1 : NULL;
-	return eb_true;
+	return true;
 }				/* parseURL */
 
-eb_bool isURL(const char *url)
+bool isURL(const char *url)
 {
 	int j = parseURL(url, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
 	if (j < 0)
-		return eb_false;
+		return false;
 	return j;
 }				/* isURL */
 
 /* non-FTP URLs are always browsable.  FTP URLs are browsable if they end with
 * a slash. */
-eb_bool isBrowseableURL(const char *url)
+bool isBrowseableURL(const char *url)
 {
 	if (isURL(url))
 		return (!memEqualCI(url, "ftp://", 6))
 		    || (url[strlen(url) - 1] == '/');
 	else
-		return eb_false;
+		return false;
 }				/* isBrowseableURL */
 
+bool isDataURI(const char *u)
+{
+	return memEqualCI(u, "data:", 5);
+}				/* isDataURI */
+
 /* Helper functions to return pieces of the URL.
  * Makes a copy, so you can have your 0 on the end.
  * Return 0 for an error, and "" if that piece is missing. */
@@ -351,7 +490,7 @@ const char *getHostURL(const char *url)
 	if (free_syntax)
 		return 0;
 	if (!s)
-		return EMPTYSTRING;
+		return emptyString;
 	if (l >= sizeof(hostbuf)) {
 		setError(MSG_DomainLong);
 		return 0;
@@ -409,9 +548,9 @@ const char *getUserURL(const char *url)
 	if (rc <= 0)
 		return 0;
 	if (free_syntax)
-		return EMPTYSTRING;
+		return emptyString;
 	if (!s)
-		return EMPTYSTRING;
+		return emptyString;
 	if (l >= sizeof(buf)) {
 		setError(MSG_UserNameLong2);
 		return 0;
@@ -430,9 +569,9 @@ const char *getPassURL(const char *url)
 	if (rc <= 0)
 		return 0;
 	if (free_syntax)
-		return EMPTYSTRING;
+		return emptyString;
 	if (!s)
-		return EMPTYSTRING;
+		return emptyString;
 	if (l >= sizeof(buf)) {
 		setError(MSG_PasswordLong2);
 		return 0;
@@ -464,7 +603,10 @@ void getDirURL(const char *url, const char **start_p, const char **end_p)
 		if (*--dir != '/')
 			i_printfExit(MSG_BadDirSlash, url);
 	}
-	end = strpbrk(dir, "#?\1");
+	if (*dir == '#')	/* special case */
+		end = dir;
+	else
+		end = strpbrk(dir, "?\1");
 	if (!end)
 		end = dir + strlen(dir);
 	while (end > dir && end[-1] != '/')
@@ -479,14 +621,68 @@ slash:
 	*end_p = myslash + 1;
 }				/* getDirURL */
 
-eb_bool getPortLocURL(const char *url, const char **portloc, int *port)
+/* #tag is only meaningfull after the last slash */
+char *findHash(const char *s)
+{
+	const char *t = strrchr(s, '/');
+	if (t)
+		s = t;
+	return (char *)strchr(s, '#');
+}				/* findHash */
+
+/* extract the file piece of a pathname or url */
+/* This is for debugPrint or w/, so could be chopped for convenience */
+char *getFileURL(const char *url, bool chophash)
+{
+	const char *s;
+	const char *e;
+	s = strrchr(url, '/');
+	if (s)
+		++s;
+	else
+		s = url;
+	e = 0;
+	if (isURL(url)) {
+		chophash = true;
+		e = strpbrk(s, "?\1");
+	}
+	if (!e)
+		e = s + strlen(s);
+	if (chophash) {
+		const char *h = findHash(s);
+		if (h)
+			e = h;
+	}
+/* if slash at the end then back up to the prior slash */
+	if (e == s && s > url) {
+		while (s > url && s[-1] == '/')
+			--s;
+		e = s;
+		while (s > url && s[-1] != '/')
+			--s;
+	}
+/* don't retain the .browse suffix on a url */
+	if (e - s > 7 && stringEqual(e - 7, ".browse"))
+		e -= 7;
+	if (e - s > 64)
+		e = s + 64;
+	if (e == s)
+		strcpy(hostbuf, "/");
+	else {
+		strncpy(hostbuf, s, e - s);
+		hostbuf[e - s] = 0;
+	}
+	return hostbuf;
+}				/* getFileURL */
+
+bool getPortLocURL(const char *url, const char **portloc, int *port)
 {
 	int rc = parseURL(url, 0, 0, 0, 0, 0, 0, 0, 0, portloc, port, 0, 0, 0);
 	if (rc <= 0)
-		return eb_false;
+		return false;
 	if (free_syntax)
-		return eb_false;
-	return eb_true;
+		return false;
+	return true;
 }				/* getPortLocURL */
 
 int getPortURL(const char *url)
@@ -500,27 +696,11 @@ int getPortURL(const char *url)
 	return port;
 }				/* getPortURL */
 
-eb_bool isProxyURL(const char *url)
+bool isProxyURL(const char *url)
 {
 	return ((url[0] | 0x20) == 'p');
 }
 
-/*
- * hasPrefix: return true if s has a prefix of p, false otherwise.
- */
-static eb_bool hasPrefix(char *s, char *p)
-{
-	eb_bool ret = eb_false;
-	if (!p[0])
-		ret = eb_true;	/* Empty string is a prefix of all strings. */
-	else {
-		size_t slen = strlen(s);
-		size_t plen = strlen(p);
-		ret = (plen <= slen) && !strncmp(p, s, plen);
-	}
-	return ret;
-}				/* hasPrefix */
-
 /*
  * copyPathSegment: copy everything from *src, starting with the leftmost
  * character (a slash), and ending with either the next slash (not included)
@@ -566,9 +746,13 @@ static void squashDirectories(char *url)
 	if (strchr("#?\1", *dd))
 		return;
 	--dd;
+/* dd could point to : in bogus code such as <A href=crap:foobar> */
+/* crap: looks like a slashless protocol, perhaps unknown to us. */
+	if (*dd == ':')
+		return;
 	if (*dd != '/')
 		i_printfExit(MSG_BadSlash, url);
-	end = dd + strcspn(dd, "?#\1");
+	end = dd + strcspn(dd, "?\1");
 	rest = cloneString(end);
 	inPath = pullString1(dd, end);
 	s = inPath;
@@ -577,13 +761,13 @@ static void squashDirectories(char *url)
 /* We can ignore several steps because of a loop invariant: */
 /* After the test, *s is always a slash. */
 	while (*s) {
-		if (hasPrefix(s, "/./"))
+		if (!strncmp(s, "/./", 3))
 			s += 2;	/* Point s at 2nd slash */
 		else if (!strcmp(s, "/.")) {
 			s[1] = '\0';
 			/* We'll copy the segment "/" on the next iteration. */
 			/* And that will be the final iteration of the loop. */
-		} else if (hasPrefix(s, "/../")) {
+		} else if (!strncmp(s, "/../", 4)) {
 			s += 3;	/* Point s at 2nd slash */
 			snipLastSegment(&outPath, &outPathLen);
 		} else if (!strcmp(s, "/..")) {
@@ -608,13 +792,18 @@ char *resolveURL(const char *base, const char *rel)
 	char *q;
 	int l;
 
+	if (memEqualCI(rel, "data:", 5))
+		return cloneString(rel);
+
 	if (!base)
-		base = EMPTYSTRING;
-	n = allocMem(strlen(base) + strlen(rel) + 12);
+		base = emptyString;
+	n = allocString(strlen(base) + strlen(rel) + 12);
 	debugPrint(5, "resolve(%s|%s)", base, rel);
 
-	if (rel[0] == '#') {
-/* this is always an anchor for the current document, don't resolve. */
+	if (rel[0] == '#' && !strchr(rel, '/')) {
+/* This is an anchor for the current document, don't resolve. */
+/* I assume the base does not have a #fragment on the end; that is not part of the base. */
+/* Thus I won't get url#foo#bar */
 		strcpy(n, rel);
 out_n:
 		debugPrint(5, "= %s", n);
@@ -622,11 +811,13 @@ out_n:
 	}
 
 	if (rel[0] == '?' || rel[0] == '\1') {
+/* setting or changing get or post data */
 		strcpy(n, base);
-		for (q = n; *q && *q != '\1' && *q != '#' && *q != '?'; q++) ;
+		for (q = n; *q && *q != '\1' && *q != '?'; q++) ;
 		strcpy(q, rel);
 		goto out_n;
 	}
+
 	if (rel[0] == '/' && rel[1] == '/') {
 		if (s = strstr(base, "//")) {
 			strncpy(n, base, s - base);
@@ -636,7 +827,9 @@ out_n:
 		strcat(n, rel);
 		goto squash;
 	}
+
 	if (parseURL(rel, &s, &l, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0) > 0) {
+/* has a protocol */
 		n[0] = 0;
 		if (s != rel) {
 /* It didn't have http in front of it before, put it on now. */
@@ -645,7 +838,8 @@ out_n:
 		}
 		strcat(n, rel);
 		goto squash;
-	}			/* relative is already a url */
+	}
+	/* relative is already a url */
 	s = base;
 	if (rel[0] == '/') {
 		s = getDataURL(base);
@@ -674,7 +868,7 @@ out_n:
 	for (p = 0; *s; ++s) {
 		if (*s == '/')
 			p = s;
-		if (strchr("#?\1", *s))
+		if (strchr("?\1", *s))
 			break;
 	}
 	if (!p) {
@@ -689,60 +883,82 @@ out_n:
 		n[l++] = '/';
 	}
 	strcpy(n + l, rel);
+
 squash:
 	squashDirectories(n);
 	goto out_n;
 }				/* resolveURL */
 
 /* This routine could be, should be, more sophisticated */
-eb_bool sameURL(const char *s, const char *t)
+bool sameURL(const char *s, const char *t)
 {
-	const char *u, *v;
+	const char *u, *p, *q;
 	int l;
+
+	if (!s || !t)
+		return false;
+
+/* check for post data at the end */
+	p = strchr(s, '\1');
+	if (!p)
+		p = s + strlen(s);
+	q = strchr(t, '\1');
+	if (!q)
+		q = t + strlen(t);
+	if (!stringEqual(p, q))
+		return false;
+
+/* lop off hash */
+	if (u = findHash(s))
+		p = u;
+	if (u = findHash(t))
+		q = u;
+
 /* It's ok if one says http and the other implies it. */
 	if (memEqualCI(s, "http://", 7))
 		s += 7;
 	if (memEqualCI(t, "http://", 7))
 		t += 7;
-	u = s + strcspn(s, "#");
-	v = t + strcspn(t, "#?\1");
-	if (u - s >= 7 && stringEqual(u - 7, ".browse"))
-		u -= 7;
-	if (v - t >= 7 && stringEqual(v - 7, ".browse"))
-		v -= 7;
-	l = u - s;
-	if (l != v - t)
-		return eb_false;
+
+	if (p - s >= 7 && stringEqual(p - 7, ".browse"))
+		p -= 7;
+	if (q - t >= 7 && stringEqual(q - 7, ".browse"))
+		q -= 7;
+	l = p - s;
+	if (l != q - t)
+		return false;
 	return !memcmp(s, t, l);
 }				/* sameURL */
 
-/* Find some helpful text to print, in place of an image. */
-/* Text longer than 80 chars isn't helpful, so we return a static buffer. */
+/* Find some helpful text to print, in place of an image.
+ * Not sure why we would need more than 1000 chars for this,
+ * so return a static buffer. */
 char *altText(const char *base)
 {
-	static char buf[80];
+	static char buf[1000];
 	int len, n;
 	int recount = 0;
 	char *s;
-	debugPrint(5, "altText(%s)", base);
+	debugPrint(6, "altText(%s)", base);
 	if (!base)
 		return 0;
 	if (stringEqual(base, "#"))
 		return 0;
 	if (memEqualCI(base, "javascript", 10))
 		return 0;
+
 retry:
 	if (recount >= 2)
 		return 0;
 	strncpy(buf, base, sizeof(buf) - 1);
-	spaceCrunch(buf, eb_true, eb_false);
+	spaceCrunch(buf, true, false);
 	len = strlen(buf);
 	if (len && !isalnumByte(buf[len - 1]))
 		buf[--len] = 0;
 	while (len && !isalnumByte(buf[0]))
 		strmove(buf, buf + 1), --len;
 	if (len > 10) {
-/* see if it's a phrase/sentence or a pathname/url */
+/* see whether it's a phrase/sentence or a pathname/url */
 /* Do this by counting spaces */
 		for (n = 0, s = buf; *s; ++s)
 			if (*s == ' ')
@@ -750,8 +966,8 @@ retry:
 		if (n * 8 >= len)
 			return buf;	/* looks like words */
 /* Ok, now we believe it's a pathname or url */
-/* Get rid of everything after ? or # */
-		s = strpbrk(buf, "#?\1");
+/* get rid of post or get data */
+		s = strpbrk(buf, "?\1");
 		if (s)
 			*s = 0;
 /* get rid of common suffix */
@@ -807,16 +1023,12 @@ char *encodePostData(const char *s)
 
 	if (!s)
 		return 0;
-	if (s == EMPTYSTRING)
-		return EMPTYSTRING;
+	if (s == emptyString)
+		return emptyString;
 	post = initString(&l);
 	while (c = *s++) {
 		if (isalnumByte(c))
 			goto putc;
-		if (c == ' ') {
-			c = '+';
-			goto putc;
-		}
 		if (strchr("-._~*()!", c))
 			goto putc;
 		sprintf(buf, "%%%02X", (uchar) c);
@@ -850,7 +1062,7 @@ static char dohex(char c, const char **sp)
 char *decodePostData(const char *data, const char *name, int seqno)
 {
 	const char *s, *n, *t;
-	char *new = 0, *w = 0;
+	char *ns = 0, *w = 0;
 	int j = 0;
 	char c;
 
@@ -865,7 +1077,7 @@ char *decodePostData(const char *data, const char *name, int seqno)
 /* select attribute by number */
 		++j;
 		if (j == seqno)
-			w = new = allocMem(t - s + 1);
+			w = ns = allocString(t - s + 1);
 		if (seqno && !w)
 			continue;
 		if (name)
@@ -895,7 +1107,7 @@ char *decodePostData(const char *data, const char *name, int seqno)
 			if (name)
 				continue;
 			*w = 0;
-			return new;
+			return ns;
 		}
 		if (w)
 			*w++ = c;
@@ -905,7 +1117,7 @@ char *decodePostData(const char *data, const char *name, int seqno)
 				continue;
 			if (*n)
 				continue;
-			w = new = allocMem(t - s + 1);
+			w = ns = allocString(t - s + 1);
 		}
 
 /* At this point we have a match */
@@ -915,7 +1127,7 @@ char *decodePostData(const char *data, const char *name, int seqno)
 			*w++ = c;
 		}
 		*w = 0;
-		return new;
+		return ns;
 	}
 
 	return 0;
@@ -924,7 +1136,6 @@ char *decodePostData(const char *data, const char *name, int seqno)
 void decodeMailURL(const char *url, char **addr_p, char **subj_p, char **body_p)
 {
 	const char *s;
-	char *new;
 	if (memEqualCI(url, "mailto:", 7))
 		url += 7;
 	s = url + strcspn(url, "/?");
@@ -943,120 +1154,3 @@ void decodeMailURL(const char *url, char **addr_p, char **subj_p, char **body_p)
 	if (body_p)
 		*body_p = decodePostData(url, "body", 0);
 }				/* decodeMailURL */
-
-/*********************************************************************
-Given a protocol and a domain, find the proxy server
-to mediate your request.
-This is the C version, using entries in .ebrc.
-There is a javascript version of the same name, that we will support later.
-This is a beginning, and it can be used even when javascript is disabled.
-A return of null means DIRECT, and this is the default
-if we don't match any of the proxy entries.
-*********************************************************************/
-
-static const char *findProxyInternal(const char *prot, const char *domain)
-{
-	struct PXENT *px = proxyEntries;
-	int i;
-
-/* first match wins */
-	for (i = 0; i < maxproxy; ++i, ++px) {
-
-		if (px->prot) {
-			char *s = px->prot;
-			char *t;
-			int rc;
-			while (*s) {
-				t = strchr(s, '|');
-				if (t)
-					*t = 0;
-				rc = stringEqualCI(s, prot);
-				if (t)
-					*t = '|';
-				if (rc)
-					goto domain;
-				if (!t)
-					break;
-				s = t + 1;
-			}
-			continue;
-		}
-
-domain:
-		if (px->domain) {
-			int l1 = strlen(px->domain);
-			int l2 = strlen(domain);
-			if (l1 > l2)
-				continue;
-			l2 -= l1;
-			if (!stringEqualCI(px->domain, domain + l2))
-				continue;
-			if (l2 && domain[l2 - 1] != '.')
-				continue;
-		}
-
-		return px->proxy;
-	}
-
-	return 0;
-}				/* findProxyInternal */
-
-const char *findProxyForURL(const char *url)
-{
-	return findProxyInternal(getProtURL(url), getHostURL(url));
-}				/* findProxyForURL */
-
-static char **novs_hosts;
-size_t novs_hosts_avail;
-size_t novs_hosts_max;
-
-void addNovsHost(char *host)
-{
-	if (novs_hosts_max == 0) {
-		novs_hosts_max = 32;
-		novs_hosts = allocZeroMem(novs_hosts_max * sizeof(char *));
-	} else if (novs_hosts_avail >= novs_hosts_max) {
-		novs_hosts_max *= 2;
-		novs_hosts = reallocMem(novs_hosts, novs_hosts_max * sizeof(char *));
-	}
-	novs_hosts[novs_hosts_avail++] = host;
-}				/* addNovsHost */
-
-/* Return true if the cert for this host should be verified. */
-static eb_bool mustVerifyHost(const char *host)
-{
-	size_t this_host_len = strlen(host);
-	size_t i;
-
-	if (!verifyCertificates)
-		return eb_false;
-
-	for (i = 0; i < novs_hosts_avail; i++) {
-		size_t l1 = strlen(novs_hosts[i]);
-		size_t l2 = this_host_len;
-		if (l1 > l2)
-			continue;
-		l2 -= l1;
-		if (!stringEqualCI(novs_hosts[i], host + l2))
-			continue;
-		if (l2 && host[l2 - 1] != '.')
-			continue;
-		return eb_false;
-	}
-	return eb_true;
-}				/* mustVerifyHost */
-
-CURLcode setCurlURL(CURL * h, const char *url)
-{
-	const char *proxy = findProxyForURL(url);
-	if (!proxy)
-		proxy = "";
-	else
-		debugPrint(3, "proxy %s", proxy);
-	const char *host = getHostURL(url);
-	unsigned long verify = mustVerifyHost(host);
-	curl_easy_setopt(h, CURLOPT_PROXY, proxy);
-	curl_easy_setopt(h, CURLOPT_SSL_VERIFYPEER, verify);
-	curl_easy_setopt(h, CURLOPT_SSL_VERIFYHOST, verify);
-	return curl_easy_setopt(h, CURLOPT_URL, url);
-}				/* setCurlURL */
